/*-------------------------------------------------------------------------*
 *
 * FileSys.h
 *  This file defines the file system API.
 *
 * Copyright 1997-1998 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/

#ifndef __FILESYS_H
#define __FILESYS_H

#ifdef __cplusplus
extern "C" {
#endif

#include "BaseType.h"


/*-------------------------------------------------------------------------*
 * Constants.
 *-------------------------------------------------------------------------*/

/* Number of record handles. */
#define NUM_HANDLE 6144

/* Number of file handles. */
#define NUM_FILE   10

/* Return codes. */
#define DB_OK                   0       // Everything is OK.

#define DB_NO_DB                -1      // There is not such data base, not an error.
#define DB_NO_REC               -2      // There is not such record, not an error.

#define DB_ERR_NO_SPACE         -10     // No room left in file system.
#define DB_ERR_NO_HANDLE        -11     // Too many records in use; no free handles.
#define DB_ERR_NOT_EXIST        -12     // The named DB doesn't exist.
#define DB_ERR_BAD_HANDLE       -13     // The specified handle is not valid.
#define DB_ERR_BAD_SIZE         -14     // The size record is invalid
#define DB_ERR_BOUNDS           -15     // The region exceeds the bounds of the record.
#define DB_ERR_WRONG_DB         -16     // The database did not match the record.
#define DB_ERR_NOT_DIR          -17     // The handle didn't point to a directory entery.
#define DB_ERR_IS_DIR           -18     // The handle pointed to a directory entery.
#define DB_ERR_NOT_ORPHAN       -19     //
#define DB_ERR_BAD_NAME         -20     // The specified name is invalid.
#define DB_ERR_PRIV_HANDLE      -21     // The specified handle is privileged.

#define DB_ERR_FILE_OPEN        -100    //
#define DB_ERR_NO_FILE          -101    //
#define DB_ERR_BAD_FILE         -102    //
#define DB_ERR_FILE_CLOSED      -103    //
#define DB_ERR_TYPE_MATCH       -104    //


/*-------------------------------------------------------------------------*
 * Types.
 *-------------------------------------------------------------------------*/

typedef char    FileType;
typedef short   HandleType;
typedef int     STATUS;

typedef enum {NORMAL, ORPHAN, DB_NAME} RecKind;

typedef struct {
    void       *Disk;
    unsigned    NumOfBlock;
    unsigned    BlockSize;
    unsigned    NumOfClean;
} FileSysInfoType;

typedef struct {
    unsigned    Pos;
    unsigned    Length;
    HandleType  Db;
} FileInfoType;

/*-------------------------------------------------------------------------*
 * Function Prototypes.
 *-------------------------------------------------------------------------*/

#ifdef __cplusplus
    STATUS      __cdecl DbDelete(const char *fileName);
    void        __cdecl DbFileSysInfo(FileSysInfoType &FileSysInfo);
    HandleType  __cdecl DbFindNext(HandleType db, const char *pattern);
    HandleType  __cdecl DbGetHandle(const char *fileName);
    const char *__cdecl DbName(HandleType db);
    const void *const *__cdecl DbPointTable();
#else
    STATUS      __cdecl DbDelete(char *fileName);
    void        __cdecl DbFileSysInfo(FileSysInfoType *FileSysInfo);
    HandleType  __cdecl DbFindNext(HandleType db, char *pattern);
    HandleType  __cdecl DbGetHandle(char *fileName);
    char       *__cdecl DbName(HandleType db);
    void      **__cdecl DbPointTable();
#endif

STATUS      __cdecl DbAddOrphan(HandleType db, HandleType orphan);
HandleType  __cdecl DbAddRec(HandleType db, unsigned size, const void *data);
STATUS      __cdecl DbAndRec(HandleType rec, const void *mask, unsigned size, unsigned offSet);
STATUS      __cdecl DbDeleteRec(HandleType db, HandleType rec);
HandleType  __cdecl DbFirstRec(HandleType db);
unsigned    __cdecl DbFreeRec();
unsigned    __cdecl DbFreeSpace();
unsigned    __cdecl DbMaxNewRecSize(RecKind NewRec);
HandleType  __cdecl DbNextRec(HandleType prevRec);
int         __cdecl DbRecSize(HandleType rec, BOOL dataOnly);
STATUS      __cdecl DbReplaceOrphan(HandleType rec, HandleType orphan);
STATUS      __cdecl DbReplaceRec(HandleType rec, unsigned newSize, const void *data);
void        __cdecl DbSecure();
int         __cdecl DbSize(HandleType db, BOOL dataOnly);

STATUS      __cdecl DbFileClose(FileType file, BOOL trim);
BOOL        __cdecl DbFileInfo(FileType file, FileInfoType *fileInfo);
FileType    __cdecl DbFileOpen(HandleType db, unsigned newRecSize);
int         __cdecl DbFileRead(FileType file, void *data, unsigned size);
STATUS      __cdecl DbFileSeek(FileType file, unsigned pos);
STATUS      __cdecl DbFileWrite(FileType file, const void *data, unsigned size);

#ifdef __cplusplus
    #pragma warning( disable:4514 ) // disable warning about unused inline function
    inline const void *HandleToPoint(HandleType Handle, const void *const *PointTable)
		{ return PointTable[Handle]; };
    inline BOOL ValidHandle(HandleType Handle, const void *const *PointTable)
		{ return ((0 <= Handle) && (Handle < NUM_HANDLE) && (PointTable[Handle] != NULL)); };
#else
    #define HandleToPoint(Handle,PointTable) PointTable[Handle]
    #define ValidHandle(Handle,PointTable) ((0 <= Handle) && (Handle < NUM_HANDLE) && (PointTable[Handle] != NULL))
#endif


#ifdef __cplusplus
}
#endif

#endif  /* __FILESYS_H */
