/*
 * bitfield.h
 *
 * Turn on function inlining optimizations!
 */
 
#ifndef BITFIELD_H
#define BITFIELD_H

 
#include <memory.h>

#include "utilities.h"


#define NUM_BITS_PER_INT  ( sizeof( unsigned int ) * 8 )
#define NUM_BYTES_PER_INT ( sizeof( unsigned int ) )

#define BITFIELD_NUM_INTS(  bits ) ( ( ( bits ) + NUM_BITS_PER_INT - 1 ) / NUM_BITS_PER_INT )
#define BITFIELD_NUM_BYTES( bits ) ( BITFIELD_NUM_INTS( bits ) * NUM_BYTES_PER_INT )



extern const int first_bit[ 37 ];

                               

class BitField 
{
   
   protected:
      
     unsigned int * bitfield;
     
     int num_bits;
     int num_bytes;
     int num_ints;

     int num_bits_in_use;
      
      
   public:
   
      BitField( int init_num_bits_in_use,
                unsigned int * init_bitfield,
                int init_num_bytes )
         : bitfield( init_bitfield )
         , num_bits( init_num_bytes * 8 )
         , num_bytes( init_num_bytes )
         , num_ints( init_num_bytes / NUM_BYTES_PER_INT )
         , num_bits_in_use( init_num_bits_in_use )
      {
         clear_all();
         }


      int
      get_num_bits( void ) const
      {
         return( num_bits_in_use );
         }


      void
      set_all( void ) 
      {
         memset( bitfield, 0xff, num_bytes );
         }
      
      
      void                       
      set( int bit )
      {
         bitfield[ calculate_index( bit ) ] |= set_mask( bit );
         }
         
         
      bool
      is_set( int bit ) const
      {
         return( ( bitfield[ calculate_index( bit ) ] & set_mask( bit ) ) ? true : false );
         }
         
         
      void
      clear_all( void )
      {
         memset( bitfield, 0x00, num_bytes );
         }
      
      
      void
      clear( int bit )
      {
         bitfield[ calculate_index( bit ) ] &= clear_mask( bit );
         }
         
         
      bool
      is_clear( int bit ) const
      {
         return( ( bitfield[ calculate_index( bit ) ] & set_mask( bit ) ) ? false : true );
         }
         

      virtual
      bool  
      insert( int bit,
              bool value );


      virtual
      bool
      grow_bitfield( int bit )
      {
         if( bit < num_bits ) {
            if( num_bits_in_use < bit ) {
               num_bits_in_use = bit;
               }
            return( true );
            }

         return( false );
         }


      virtual
      void
      shrink_bitfield( void )
      {
         }


      virtual
      void
      delete_bit( int bit,
                  bool value = 0 );


      int
      get_first_set_bit( void ) const;
      

      int
      get_nth_set_bit( int i ) const;
         
         
      int
      get_last_set_bit( void ) const; 
                                   
         
      int
      get_next_set_bit( int bit ) const; 
         
         
      int
      get_previous_set_bit( int bit ) const; 
         

      int
      get_nth_cleared_bit( int i ) const;
         
         
      int
      get_first_cleared_bit( void ) const; 
         
         
      int
      get_last_cleared_bit( void ) const; 
         
         
      int
      get_next_cleared_bit( int bit ) const; 
         
         
      int
      get_previous_cleared_bit( int bit ) const; 
         
         
      int
      count_set_bits( void ) const;


      int
      count_set_bits_before( int bit ) const;


      int
      count_set_bits_after( int bit ) const;
      
         
      int
      count_cleared_bits( void ) const;

      
      int
      count_cleared_bits_before( int bit ) const;


      int
      count_cleared_bits_after( int bit ) const;
      
         
      void
      operator = ( const BitField & rhs )
      {
         grow_bitfield( rhs.num_bits );

         memcpy( bitfield, rhs.bitfield, num_bytes );
         } 
         
         
      void
      operator &= ( const BitField & rhs )
      {
         grow_bitfield( rhs.num_bits );

         for( int i=MIN( num_ints, rhs.num_ints ) - 1; i >= 0; i-- ) {
            bitfield[i] &= rhs.bitfield[i];
            }
         } 
         
      void
      operator |= ( const BitField & rhs )
      {
         grow_bitfield( rhs.num_bits );

         for( int i=MIN( num_ints, rhs.num_ints ) - 1; i >= 0; i-- ) {
            bitfield[i] |= rhs.bitfield[i];
            }
         } 
         
      void
      operator ^= ( const BitField & rhs )
      {
         grow_bitfield( rhs.num_bits );

         for( int i=MIN( num_ints, rhs.num_ints ) - 1; i >= 0; i-- ) {
            bitfield[i] ^= rhs.bitfield[i];
            }
         } 

      
   private:

      int
      calculate_index( int bit ) const
      {
         return( bit / NUM_BITS_PER_INT );
         }


      int
      calculate_bit_position( int bit ) const
      {
         return( bit % NUM_BITS_PER_INT );
         }


      int
      calculate_bit( int index,
                     unsigned int bits ) const
      { 
         return( index * NUM_BITS_PER_INT + first_bit[ ( bits ^ (bits-1) ) % 37 ] );
         }
  

      unsigned int
      set_mask( int bit ) const
      {
         return( 1 << calculate_bit_position( bit ) );
         }


      unsigned int
      clear_mask( int bit ) const
      {
         return( ~( 1 << calculate_bit_position( bit ) ) );
         }


      void
      set_lsb( unsigned int & bits ) const
      {
         bits |= bits + 1;
         }


      void
      clear_lsb( unsigned int & bits ) const
      {
         bits &= bits - 1;
         }

       
      void
      clear_higher_bits( int bit,
                         unsigned int & bits ) const
      {
         bits &= ~( ( ( 1 << calculate_bit_position( bit ) ) << 1 ) - 1 );
         }


      void
      clear_lower_bits( int bit,
                        unsigned int & bits ) const
      {
         bits &= ( 1 << calculate_bit_position( bit ) ) - 1;
         }
      

      void
      set_lower_bits( int bit,
                      unsigned int & bits ) const
      {
         bits |= ( ( 1 << calculate_bit_position( bit ) ) << 1 ) - 1;
         }


      void
      set_higher_bits( int bit,
                       unsigned int & bits ) const
      {
         bits |= ~( ( 1 << calculate_bit_position( bit ) ) - 1 );
         }

   };



   


class DynamicBitField : public BitField   
{
   private:

      unsigned int default_bits;


   public:

      DynamicBitField( int init_default_bits = 0,
                       int init_num_bits_in_use = 0 )
         : default_bits( init_default_bits ? (unsigned int) -1 : 0 )
         , BitField( init_num_bits_in_use, NULL, 0 )
      {
         grow_bitfield( num_bits_in_use );
         }


      bool
      error( void )
      {
         return( bitfield == NULL );
         }


      bool
      grow_bitfield( int bit )
      {
         if( bit > num_bits_in_use ) {
            num_bits_in_use = bit;
            }

         if( num_bits_in_use >= num_bits ) {

            if( !realloc_bitfield( num_bytes + BITFIELD_NUM_BYTES( 256 ) ) ) {
               return( false );
               }

            }

         return( true );
         }


      void
      shrink_bitfield( void )
      {
         if( num_bits_in_use < num_bits - 512 ) {
            realloc_bitfield( num_bytes - BITFIELD_NUM_BYTES( 256 ) );
            }
         }


      bool
      realloc_bitfield( int new_num_bytes );


   };


#endif
