/*
 * darray.h
 *
 * Dynamic Array Class
 */
 
#ifndef DYNAMIC_ARRAY_H
#define DYNAMIC_ARRAY_H


#include <string.h>

#include "basetype.h"
#include "utilities.h"



const int ARRAY_DELTA = 25;

template< class T >
class DynamicArray
{

   private:
   
      T ** array_pptr;

      T ** flash_array_pptr;
      HandleType db_database_handle;
      HandleType db_record_handle;

      T * ram_array;

      int num_entries;
      int max_num_entries;
      
      bool loaded_flag;
      
      int cursor;

      T default_entry;
     
   
   public:
      
      DynamicArray( void )
         : array_pptr( &ram_array )
         , flash_array_pptr( NULL )
         , db_database_handle( -1 )
         , db_record_handle( -1 )
         , ram_array( NULL )
         , num_entries( 0 )
         , max_num_entries( 0 )
         , loaded_flag( true )
         , cursor( -1 )
      {
         }


      DynamicArray( const T & init_default_entry,
                    int init_num_entries = 0,
                    bool use_flash_flag = false,
                    char * init_filename = NULL,
                    HandleType init_db_record_handle = -1,
                    bool create_flag = true )
         : array_pptr( &ram_array )
         , flash_array_pptr( NULL )
         , db_database_handle( -1 )
         , db_record_handle( -1 )
         , ram_array( NULL )
         , num_entries( 0 )
         , max_num_entries( 0 )
         , loaded_flag( true )
         , cursor( -1 )
      {
         initialize( init_default_entry, 
                     init_num_entries, 
                     use_flash_flag, 
                     init_filename,
                     init_db_record_handle,
                     create_flag );
         }


      void
      initialize( const T & init_default_entry,
                  int init_num_entries = 0,
                  bool use_flash_flag = false,
                  char * filename = NULL,
                  HandleType init_db_record_handle = -1,
                  bool create_flag = true );


      bool
      error( void ) const
      {
         return( db_record_handle < 0 );
         }
         
        
      ~DynamicArray()
      {
         if( ram_array != NULL ) {
            RimFree( ram_array );
            }

         if( db_database_handle < 0 && db_record_handle >= 0 ) {
            DbDeleteRec( db_database_handle, db_record_handle );
            }
         }
         
         
      void
      reset( void )
      {
         if( ram_array != NULL ) {
            RimFree( ram_array );
            ram_array = NULL;
            }

         num_entries = 0;
         max_num_entries = 0;
         loaded_flag = true;
         }


      bool
      load_array( int index = 0 )
      {
         return( loaded_flag || load( index ) );
         }
         
                                     
      bool
      save_array( bool free_ram_flag = true );
         
                                     
      bool
      copy_array( const DynamicArray< T > & original_array )
      {
         if( grow_array( original_array.num_entries ) ) {
            num_entries = original_array.num_entries;
            memcpy( ram_array, original_array.ram_array, sizeof( T ) * num_entries );
            return( true );
            }
            
         return( false );
         }


      int
      get_num_entries( void ) const
      {
         return( num_entries );
         }


      void
      set_num_entries( int new_num_entries )
      {
         num_entries = new_num_entries;
         }

         
      bool
      grow_array( int index )
      {
         return( index < max_num_entries || grow( index + ARRAY_DELTA ) );
         }
         
        
      void
      shrink_array( void )
      {
         if( num_entries + 2 * ARRAY_DELTA < max_num_entries ) {
            grow( num_entries + ARRAY_DELTA );
            }
         }
            
         
      int
      find_entry( const T & entry ) const
      {
         int i = num_entries - 1;
         const T * array = get_array_ptr();

         while( i>=0 && entry != array[i] ) {
            i--;
            }
         
         return( i );
         }
      
      
      bool
      insert( int index, 
              const T & entry );
   
   
      bool
      delete_index( int index );
         
         
      bool
      delete_entry( const T & entry )
      {
         return( delete_index( find_entry( entry ) ) );
         }
         
         
      T &
      operator[]( int index ) const
      {
         return( (*array_pptr)[ index ] );
         }
         
        
      T &
      get_entry( int index ) const
      {
         return( (*array_pptr)[ index ] );
         }


      T *
      get_array_ptr( void ) const
      {
         return( *array_pptr );
         }
         
        
      int
      get_cursor( void ) const
      {
         return( cursor );
         }
         
         
      void
      set_cursor( int new_cursor )
      {
         cursor = new_cursor;
         }

      
   private:
         
      bool
      grow( int index );
        

      bool
      load( int index );
         
                                     
   };
   
   
   
template< class T > 
void
DynamicArray< T >::
initialize( const T & init_default_entry,
            int init_num_entries,
            bool use_flash_flag,
            char * filename,
            HandleType init_db_record_handle,
            bool create_flag )
{
   default_entry = init_default_entry;

   if( use_flash_flag ) {

      db_database_handle = -1;
      db_record_handle   = init_db_record_handle;

      if( filename ) { 
         db_database_handle = DbGetHandle( filename );
         }

      if( db_record_handle < 0 ) {
         if( !create_flag ) {
            db_record_handle = DbFirstRec( db_database_handle );
            }
         if( db_record_handle < 0 ) {
            db_record_handle = DbAddRec( db_database_handle, 1, NULL );  // default record
            }
         }

      if( db_record_handle >= 0 ) {
         flash_array_pptr = (T **) &( DbPointTable()[ db_record_handle ] );
         }
      }

   if( init_num_entries > 0 ) {
      grow( init_num_entries );
      }
   }



template< class T > 
bool
DynamicArray< T >::
grow( int index )
{
   int new_max_num_entries;
   T * new_ram_array;
   
   new_max_num_entries = index;
   new_ram_array = (T *) RimRealloc( ram_array, 
                                     new_max_num_entries * sizeof( T ) );
   if( new_ram_array == NULL ) {
      return( false );
      }
      
   ram_array = new_ram_array;
   max_num_entries = new_max_num_entries;
   
   for( index = num_entries; index < max_num_entries; index++ ) {
      ram_array[ index ] = default_entry;
      }
   
   return( true );
   }
         

         
template< class T > 
bool
DynamicArray< T >::
insert( int index, 
        const T & entry )
{
   int new_num_entries = MAX( index, num_entries ) + 1;
   
   if( !load_array( new_num_entries ) ) {
      return( false );
      }

   if( !grow_array( new_num_entries ) ) {
      return( false );
      }

   memmove( &ram_array[ index + 1 ],
            &ram_array[ index ],
            ( max_num_entries - 1 - index ) * sizeof( T ) );
   
   ram_array[ index ] = entry;
   
   num_entries = new_num_entries;
   
   if( cursor >= index ) {
      cursor++;
      }
   
   return( true );
   }

   
   
template< class T > 
bool
DynamicArray< T >::
delete_index( int index )
{
   if( index < 0 ) {
      return( true );
      }
   
   if( !load_array() ) {
      return( false );
      }

   memmove( &ram_array[ index ],
            &ram_array[ index + 1 ],
            ( num_entries - index ) * sizeof( T ) );
   
   num_entries--;
   
   if( index <= cursor ) {
      if( index == cursor ) {
         cursor = -1;
         }
      else {
         cursor--;
         }
      }

   return( true );
   }

   
   
template< class T > 
bool
DynamicArray< T >::
load( int index )
{
   if( db_record_handle < 0 ) {
      return( false );
      }

   if( !grow( max_num_entries ) ) {      // index
      return( false );
      }

   memcpy( ram_array, 
           *flash_array_pptr, 
           max_num_entries * sizeof( T ) );

   array_pptr = &ram_array;
   loaded_flag = true;

   return( true );
   }
   

                               
template< class T > 
bool
DynamicArray< T >::
save_array( bool free_ram_flag )
{
#if 0
   if( !loaded_flag ) {
      return( true );
      }

   if( db_record_handle < 0 ) {
      return( false );
      }

   if( DbReplaceRec( db_record_handle, max_num_entries * sizeof( T ), ram_array ) < 0 ) {
      return( false );
      }

   if( free_ram_flag ) {
      RimFree( ram_array );
      ram_array = NULL;
      }

   array_pptr = flash_array_pptr;
   loaded_flag = false;
#endif
   return( true );
   }
   
                                     
   
#endif        

