/*
 * message.h
 */
 
#ifndef MESSAGE_H
#define MESSAGE_H

#include "warning.h"

#include <string.h>

#include "type.h"
#include "date.h"
#include "database.h"
#include "storage.h"

#include "PredefinedBitmaps.h"
#include "AddressFieldType.h"
#include "format_api.h"


 
class MessageFieldsDatabase : public Database
{
   public:

      MessageFieldsDatabase( char * init_name )
         : Database( init_name )
      {
         }


      void
      check_database( DbRecordBitField & fields_handle_bitfield );


   };


class MessageDatabase : public Database
{
   private:

      MessageFieldsDatabase message_fields_database;


   public:
      
      MessageDatabase( void );


      Database &
      get_message_fields_database( void ) 
      {
         return( message_fields_database );
         }


      void
      check_database( void );


      virtual
      void
      delete_record( DbRecordHandle db_record_handle );
      
      
      virtual
      void
      delete_all_records( bool zero_flag );

   };


extern MessageDatabase message_database;



 
enum MessageStatus {
   
   TX_COMPOSING      = 0x7fffffff,   // Bit 30
   TX_RETRIEVING_KEY = 0x3fffffff,   //     29
   TX_COMPRESSING    = 0x1fffffff,   //     28
   TX_ENCRYPTING     = 0x0fffffff,   //     27
   TX_PENDING        = 0x07ffffff,   //     26
   TX_SENDING        = 0x03ffffff,   //     25
   TX_SENT           = 0x01ffffff,   //     24
   TX_RETURNED       = 0x00ffffff,   //     23
   TX_MAILBOXED      = 0x007fffff,   //     22
   TX_DELIVERED      = 0x003fffff,   //     21
   TX_READ           = 0x001fffff,   //     20
   
   TX_ERROR          = 0x00001fff,   //     12
   
   RX_RECEIVING      = 0x00000fff,   //     11
   RX_UNREAD         = 0x000007ff,   //     10
   RX_READ           = 0x000003ff,   //      9
   RX_READ_ACKED     = 0x000001ff,   //      8
   
   RX_ERROR          = 0x00000001,   //      1

   };
   
   
extern char message_status_icons[ 37 ][ 2 ];
   
inline
char *
get_message_status_icon( MessageStatus message_status )
{
   return( message_status_icons[ message_status % 37 ] );
   }
   
  
extern BitMap* message_status_bitmaps[ 37 ];

inline
BitMap*
get_message_status_bitmap( MessageStatus message_status )
{
   return( message_status_bitmaps[ message_status % 37 ] );
   }
   
  
extern char * message_status_strings[ 37 ];
 
inline
char *
get_message_status_string( MessageStatus message_status )
{
   return( message_status_strings[ message_status % 37 ] );
   }
   

enum HOLSTER_TIME {
    HT_NONE = 0,
    HT__5_SECOND,
    HT_1_SECOND,
    HT_1_5_SECOND,
    HT_2_SECOND,
    HT_2_5_SECOND,
    HT_3_SECOND,
    };

   
struct MessageOptions {

   unsigned int RSVP_flag                 : 1;
   unsigned int request_receipt_ack_flag  : 1;
   unsigned int request_read_ack_flag     : 1;
   unsigned int priority_flag             : 1;

   DbRecordHandle service_record_handle;


   void
   initialize( void )
   {
      RSVP_flag = false;
      request_receipt_ack_flag = false;
      request_read_ack_flag = false;
      priority_flag = false;
      service_record_handle = DB_NULL_HANDLE;
      }
   
   };


         
struct NetworkMessageData : public FlashObjectData
{
                                                //  Transmit   Receive
   unsigned int tx_flag                  : 1;   //     x          x
   unsigned int not_saved_flag           : 1;   //     x          x
   
   unsigned int reply_flag               : 1;   //     x
   unsigned int forward_flag             : 1;   //     x

   unsigned int reply_allowed_flag       : 1;   //                x
   unsigned int no_more_available_flag   : 1;   //                x
   unsigned int no_more_requested_flag   : 1;   //                x

   unsigned int not_hidden_flag          : 1;   //     x          x       // do not show to user

   unsigned int compress_flag            : 1;   //     x          x
   unsigned int encrypt_flag             : 1;   //     x          x

   MessageStatus status;
   TransportError  error;
   
   NetworkAddress first_hop_address;

   int transport_id;
   int in_reply_to_transport_id;

   DbRecordHandle message_fields_record_handle;

   Date local_date;
   Time local_time;

   Date remote_date;
   Time remote_time;
   
   MessageOptions options;


   void
   initialize( bool transmit_flag );

   };
   
   

struct MessageAddressData 
{
   MessageStatus    status; 
   DbFieldHandle    delivery_data_field_handle;
   TransportError   error;
   AddressFieldType address_type;


   MessageAddressData( AddressFieldType init_address_type,
                       MessageStatus    init_status,
                       TransportError   init_error = 0 )
      : address_type( init_address_type )
      , status( init_status )
      , error( init_error )
      , delivery_data_field_handle( DB_INVALID_FIELD_HANDLE )
   {
      }

   };



// Address Fields
const DbFieldTag TO_FIELD_TAG             = 1;
const DbFieldTag CC_FIELD_TAG             = 2;
const DbFieldTag BCC_FIELD_TAG            = 3;
const DbFieldTag SENDER_FIELD_TAG         = 4;
const DbFieldTag FROM_FIELD_TAG           = 5;
const DbFieldTag REPLY_TO_FIELD_TAG       = 6;
const DbFieldTag MHP_FORWARD_TO_FIELD_TAG = 7;

// Simple textual fields
const DbFieldTag SUBJECT_FIELD_TAG        = 11;
const DbFieldTag MESSAGE_FIELD_TAG        = 12;
const DbFieldTag MESSAGE_ID_FIELD_TAG     = 13;
const DbFieldTag ORIGINAL_MESSAGE_ID_TAG  = 14;
const DbFieldTag DELIVERY_DATA_TAG        = 15;
const DbFieldTag TRANSPORT_ID_FIELD_TAG   = 16;
const DbFieldTag GLOBAL_DELIVERY_DATA_TAG = 17;

// Attachment fields
const DbFieldTag ATTACHMENT_FIELD_TAG     = 21;
         
// Backup and Restore fields
const DbFieldTag NETWORK_MESSAGE_DATA_FIELD_TAG = 240;
         
        


class NetworkMessage : public FlashObject
{
   private:
      
      DatabaseRecord message_fields;
      
      
   public:
   
     /*
      * read existing object
      */
      NetworkMessage( DbRecordHandle init_db_record_handle )
         : FlashObject( message_database, init_db_record_handle )
         , message_fields( message_database.get_message_fields_database(), 
                           get_data_ptr()->message_fields_record_handle )
      {
         }
         
         
     /*
      * read/write existing object
      */
      NetworkMessage( DbRecordHandle init_db_record_handle,
                      NetworkMessageData * data_ptr )
         : FlashObject( message_database, init_db_record_handle, data_ptr )
         , message_fields( message_database.get_message_fields_database(),
                           get_data_ptr()->message_fields_record_handle )
      {
         }
         
         
     /*
      * new object
      */
      NetworkMessage( NetworkMessageData * data_ptr,
                      bool transmit_flag = true )
         : FlashObject( message_database, DB_NULL_HANDLE, data_ptr )
         , message_fields( message_database.get_message_fields_database() )
      {
         NetworkMessageData & data = get_data_reference();

         data.initialize( transmit_flag );
         }


      bool
      reconstruct( DbRecordHandle db_record_handle )
      {           
         FlashObject::reconstruct( db_record_handle );

         if( db_record_handle == DB_NULL_HANDLE ) {
            get_data_reference().initialize( true );
            }
                      
         return( message_fields.reconstruct( get_data_ptr()->message_fields_record_handle ) );
         }

      
      bool
      error( void )
      {
         return( message_fields.error() );
         }

      
      virtual
      bool
      load( void )
      {
         return( load_data( sizeof( NetworkMessageData ) ) );
         }


      virtual
      bool
      load_fields( void )
      {
         return( load() && message_fields.load() );
         }


      virtual
      DbRecordHandle
      save( void )
      {
         DbRecordHandle message_fields_record_handle = message_fields.save();
         
         if( message_fields_record_handle == DB_NULL_HANDLE ) {
            return( DB_NULL_HANDLE );
            }
         
         get_data_ptr()->message_fields_record_handle = message_fields_record_handle;

         return( save_data( sizeof( NetworkMessageData ) ) );
         }


      virtual
      void
      delete_object( bool zero_flag = false )
      {
         message_fields.delete_object( zero_flag );
         FlashObject::delete_object( zero_flag );
         }


      virtual
      bool
      clear_db_record_handle( void )
      {
         return( FlashObject::clear_db_record_handle() && message_fields.clear_db_record_handle() );
         }


      virtual
      bool
      moved_in_flash( void )
      {
         return( FlashObject::moved_in_flash() || message_fields.moved_in_flash() );
         }


      bool
      get_tx_flag( void )
      {
         return( (bool) get_data_ptr()->tx_flag );
         }
         

      bool
      get_saved_flag( void )
      {
         return( ! get_data_ptr()->not_saved_flag );
         }
         
        
      void  
      set_saved_flag( bool flag )
      {
         get_data_ptr()->not_saved_flag = !flag;
         }
         
        
      bool
      get_hidden_flag( void )
      {
         return( ! get_data_ptr()->not_hidden_flag );
         }
         
        
      void  
      set_hidden_flag( void )
      {
         get_data_ptr()->not_hidden_flag = false;
         }
         
        
      bool
      get_compress_flag( void )
      {
         return( (bool) get_data_ptr()->compress_flag );
         }
         
        
      void  
      set_compress_flag( bool flag )
      {
         get_data_ptr()->compress_flag = flag;
         }
         
        
      bool
      get_encrypt_flag( void )
      {
         return( (bool) get_data_ptr()->encrypt_flag );
         }
         
        
      void  
      set_encrypt_flag( bool flag )
      {
         get_data_ptr()->encrypt_flag = flag;
         }
         
        
      MessageStatus
      get_message_status( void )
      {
         return( get_data_ptr()->status );
         }
         

      char *
      get_message_status_string( void )
      {
         return( ::get_message_status_string( get_data_ptr()->status ) );
         }
         

      char *
      get_message_status_icon( void )
      {
         return( ::get_message_status_icon( get_data_ptr()->status ) );
         }
         

      void
      set_message_status( MessageStatus new_message_status )
      {
         get_data_ptr()->status = new_message_status;
         }


      TransportError
      get_message_error( void )
      {
         return( get_data_ptr()->error );
         }
         

      char *
      get_message_error_string( void )
      {
         return( get_transport_error_string( get_data_ptr()->error ) );
         }
         

      void
      set_message_error( TransportError new_error )
      {
         get_data_ptr()->error = new_error;
         }


      Date &
      get_local_date( void )
      {
         return( get_data_ptr()->local_date );
         }
         

      Time &
      get_local_time( void )
      {
         return( get_data_ptr()->local_time );
         }
         

      Date &
      get_remote_date( void )
      {
         return( get_data_ptr()->remote_date );
         }
         

      Time &
      get_remote_time( void )
      {
         return( get_data_ptr()->remote_time );
         }


      Date &
      get_display_date( void )
      {
         #ifdef DISPLAY_LOCAL_DATE_TIME
            return( get_local_date() );
         #else
            return( get_remote_date() );
         #endif
         }


      Time &
      get_display_time( void )
      {
         #ifdef DISPLAY_LOCAL_DATE_TIME
            return( get_local_time() );
         #else
            return( get_remote_time() );
         #endif
         }


      void
      set_local_time( TIME & time )
      {
         get_data_ptr()->local_time = time;
         get_data_ptr()->local_date = time;
         }
         

      void
      set_remote_time( TIME & time )
      {
         get_data_ptr()->remote_time = time;
         get_data_ptr()->remote_date = time;
         }
         

      NetworkAddress
      get_first_hop_address( void )
      {
         return( get_data_ptr()->first_hop_address );
         }
         
       
      void  
      set_first_hop_address( NetworkAddress first_hop_address )
      {
         get_data_ptr()->first_hop_address = first_hop_address;
         }


      bool
      get_RSVP_flag( void )
      {
         return( (bool) get_data_ptr()->options.RSVP_flag );
         }
         
       
      void  
      set_RSVP_flag( bool RSVP_flag )
      {
         get_data_ptr()->options.RSVP_flag = RSVP_flag;
         }
         
        
      bool
      get_request_receipt_ack_flag( void )
      {
         return( (bool) get_data_ptr()->options.request_receipt_ack_flag );
         }
         
       
      void  
      set_request_receipt_ack_flag( bool request_receipt_ack_flag )
      {
         get_data_ptr()->options.request_receipt_ack_flag = request_receipt_ack_flag;
         }
         
        
      bool
      get_request_read_ack_flag( void )
      {
         return( (bool) get_data_ptr()->options.request_read_ack_flag );
         }
         
       
      void  
      set_request_read_ack_flag( bool request_read_ack_flag )
      {
         get_data_ptr()->options.request_read_ack_flag = request_read_ack_flag;
         }
         
        
      bool
      get_priority_flag( void )
      {
         return( (bool) get_data_ptr()->options.priority_flag );
         }
         
       
      void  
      set_priority_flag( bool priority_flag )
      {
         get_data_ptr()->options.priority_flag = priority_flag;
         }
         
        
      bool
      get_reply_flag( void )
      {
         return( (bool) get_data_ptr()->reply_flag );
         }
         
       
      void  
      set_reply_flag( bool reply_flag )
      {
         get_data_ptr()->reply_flag = reply_flag;
         }
         
        
      bool
      get_forward_flag( void )
      {
         return( (bool) get_data_ptr()->forward_flag );
         }
         
       
      void  
      set_forward_flag( bool forward_flag )
      {
         get_data_ptr()->forward_flag = forward_flag;
         }
         
        
      bool
      get_reply_allowed_flag( void )
      {
         return( (bool) get_data_ptr()->reply_allowed_flag );
         }
         
       
      void  
      set_reply_allowed_flag( bool reply_allowed_flag )
      {
         get_data_ptr()->reply_allowed_flag = reply_allowed_flag;
         }

   
      bool
      get_more_available_flag( void )
      {
         return( (bool) !get_data_ptr()->no_more_available_flag );
         }
         
       
      void  
      set_more_available_flag( bool more_available_flag )
      {
         NetworkMessageData * data_ptr = get_data_ptr();
         data_ptr->no_more_available_flag = !more_available_flag;
         data_ptr->no_more_requested_flag = true;
         }
         
        
      bool
      get_more_requested_flag( void )
      {
         return( (bool) !get_data_ptr()->no_more_requested_flag );
         }
         
       
      void  
      set_more_requested_flag( bool more_requested_flag )
      {
         get_data_ptr()->no_more_requested_flag = !more_requested_flag;
         }
         

      int
      get_transport_id( void )
      {
         return( get_data_ptr()->transport_id );
         }


      void
      set_transport_id( int new_transport_id )
      {
         get_data_ptr()->transport_id = new_transport_id;
         }


      int
      get_in_reply_to_transport_id( void )
      {
         return( get_data_ptr()->in_reply_to_transport_id );
         }


      void
      set_in_reply_to_transport_id( int new_in_reply_to_transport_id )
      {
         get_data_ptr()->in_reply_to_transport_id = new_in_reply_to_transport_id;
         }


      MessageOptions &
      get_message_options( void )
      {
         return( get_data_ptr()->options );
         }

        
     /*
      * Message fields
      */
      DbFieldHandle
      get_first_field_handle( void )
      {
         return( message_fields.get_first_field_handle() );
         }
         
         
      DbFieldHandle
      get_next_field_handle( DbFieldHandle field_handle )
      {
         return( message_fields.get_next_field_handle( field_handle ) );
         }


      DbFieldHandle
      get_first_field_handle( DbFieldTag field_tag )
      {
         return( message_fields.get_first_field_handle( field_tag ) );
         }
         
         
      DbFieldHandle
      get_next_field_handle( DbFieldHandle field_handle,
                             DbFieldTag field_tag )
      {
         return( message_fields.get_next_field_handle( field_handle, field_tag ) );
         }


      DbFieldHandle
      get_field_tag( DbFieldHandle field_handle )
      {
         return( message_fields.get_field_tag( field_handle ) );
         }


      void
      delete_field( DbFieldHandle field_handle )
      {
         message_fields.delete_field( field_handle );
         }


      void
      delete_fields( DbFieldTag field_tag )
      {
         message_fields.delete_fields( field_tag );
         }


      void * const
      get_field_data_ptr( DbFieldHandle field_handle )
      {
         return( message_fields.get_field_data_ptr( field_handle ) );
         }


      int
      get_field_data_length( DbFieldHandle field_handle )
      {
         return( message_fields.get_field_data_length( field_handle ) );
         }


     /*
      * textual message fields
      */
      char *
      get_field_text( DbFieldHandle field_handle )
      {
         return( (char *) message_fields.get_field_data_ptr( field_handle ) );
         }


      DbFieldHandle
      set_field_text( DbFieldHandle field_handle,
                      char const * const text,
                      int text_length = DB_TEXT )
      {
         return( message_fields.update_field( field_handle, 
                                              text, text_length ) );
         }                  
     

      char *
      get_tag_text( DbFieldTag field_tag )
      {
         return( (char *) message_fields.get_field_data_ptr( get_first_field_handle( field_tag ) ) );
         }


      DbFieldHandle
      set_tag_text( DbFieldTag field_tag,
                    char const * const text,
                    int text_length = DB_TEXT )
      {
         return( message_fields.update_field( message_fields.get_first_field_handle( field_tag ), 
                                              field_tag, 
                                              text, text_length ) );
         }                  
     

      DbFieldHandle
      create_field( DbFieldTag field_tag,
                    char const * const text = NULL,
                    int text_length = DB_TEXT )
      {
         return( message_fields.add_field( field_tag, text, text_length ) );
         }


      bool
      append_field_text( DbFieldHandle field_handle,
                         char const * const text,
                         int text_length = DB_TEXT )
      {
         return( message_fields.append_to_field( field_handle, text, text_length ) );
         }
 

      bool
      append_tag_text( DbFieldTag field_tag,
                       char const * const text,
                       int text_length = DB_TEXT )
      {
         return( append_field_text( message_fields.get_first_field_handle( field_tag ),
                                    text, text_length ) );
         }


      bool
      prepend_field_text( DbFieldHandle field_handle,
                          char const * const text,
                          int text_length = DB_TEXT )
      {
         return( message_fields.prepend_to_field( field_handle, text, text_length ) );
         }
 

      bool
      prepend_tag_text( DbFieldTag field_tag,
                        char const * const text,
                        int text_length = DB_TEXT )
      {
         return( prepend_field_text( message_fields.get_first_field_handle( field_tag ),
                                     text, text_length ) );
         }


      bool
      replace_field_data( DbFieldHandle field_handle,
                          int           offset,
                          const void *  data_ptr,
                          int           data_length = DB_TEXT )
      {
         return( message_fields.replace_field_data( field_handle,
                                                    offset,
                                                    data_ptr, data_length ) );
         }


      bool
      replace_tag_data( DbFieldTag    field_tag,
                        int           offset,
                        const void *  data_ptr,
                        int           data_length = DB_TEXT )
      {
         return( replace_field_data( message_fields.get_first_field_handle( field_tag ),
                                     offset,
                                     data_ptr, data_length ) );
         }



     /*
      * address fields
      */
      AddressFieldType
      get_address_type( DbFieldHandle field_handle )
      {
         return( get_address_data( field_handle )->address_type );
         }


      MessageStatus
      get_address_status( DbFieldHandle field_handle )
      {
         return( get_address_data( field_handle )->status );
         }


      TransportError
      get_address_error( DbFieldHandle field_handle )
      {
         return( get_address_data( field_handle )->error );
         }


      char *
      get_address_error_string( DbFieldHandle field_handle )
      {
         return( get_transport_error_string( get_address_data( field_handle )->error ) );
         }


      char *
      get_delivery_data( DbFieldHandle field_handle )
      {
         DbFieldHandle delivery_data_field_handle = get_address_data( field_handle )->delivery_data_field_handle;
         return( (char *) ( message_fields.get_field_data_ptr( delivery_data_field_handle ) ) );
         }


      char *
      get_address_name( DbFieldHandle field_handle )
      {
         return( (char *) ( get_address_data( field_handle ) + 1 ) );
         }
         
         
      char *
      get_address( DbFieldHandle field_handle )
      {
         char * name = get_address_name( field_handle );
         return( name + strlen( name ) + 1 );
         }


      DbFieldHandle
      create_address( DbFieldTag field_tag,
                      char const * const address,
                      MessageAddressData & address_data,
                      char const * const name = "",
                      DbFieldHandle field_handle = DB_INVALID_FIELD_HANDLE,
                      int address_length = DB_TEXT,
                      int name_length = DB_TEXT )
      {
         return( message_fields.update_field( field_handle,
                                              field_tag,
                                              &address_data, sizeof( address_data ),
                                              name, name_length,
                                              address, address_length ) );
         }


      DbFieldHandle
      create_address( DbFieldTag field_tag,
                      char const * const address,
                      AddressFieldType   address_type,
                      char const * const name = "",
                      DbFieldHandle field_handle = DB_INVALID_FIELD_HANDLE,
                      int address_length = DB_TEXT,
                      int name_length = DB_TEXT )
      {
         MessageAddressData address_data( address_type, get_message_status() );

         return( create_address( field_tag,
                                 address, address_data,
                                 name,
                                 field_handle,
                                 address_length, 
                                 name_length ) );
         }


      DbFieldHandle
      set_address( DbFieldHandle field_handle,
                   char const * const address,
                   MessageAddressData & address_data,
                   char const * const name = "",
                   int address_length = DB_TEXT,
                   int name_length = DB_TEXT )
      {
         return( message_fields.update_field( field_handle,
                                              &address_data, sizeof( address_data ),
                                              name, name_length,
                                              address, address_length ) );
         }


      DbFieldHandle
      set_address( DbFieldHandle field_handle,
                   char const * const address,
                   AddressFieldType address_type,
                   char const * const name = "",
                   int address_length = DB_TEXT,
                   int name_length = DB_TEXT )
      {
         MessageAddressData address_data( address_type, get_message_status() );

         return( message_fields.update_field( field_handle,
                                              &address_data, sizeof( address_data ),
                                              name, name_length,
                                              address, address_length ) );
         }             


      bool
      set_name( DbFieldHandle field_handle,
                char const * const name,
                int name_length = DB_TEXT )
      {
         MessageAddressData * address_data_ptr = get_address_data( field_handle );
         char * address = get_address( field_handle );

         return( message_fields.update_field( field_handle,
                                              address_data_ptr, sizeof( *address_data_ptr ),
                                              name, name_length,
                                              address, DB_TEXT ) != DB_INVALID_FIELD_HANDLE );
         }


      bool
      set_address_status( DbFieldHandle address_field_handle,
                          MessageStatus new_address_status,
                          TransportError new_error,
                          char * delivery_data = NULL,
                          int delivery_data_length = DB_TEXT )
      {
         MessageAddressData * address_data_ptr = get_address_data( address_field_handle );

         address_data_ptr->status = new_address_status;
         address_data_ptr->error  = new_error;

         set_message_status( calculate_message_status() );

         if( delivery_data == NULL || delivery_data_length == 0 ) {
            return( true );
            }

         DbFieldHandle delivery_data_field_handle;

         delivery_data_field_handle = message_fields.update_field( address_data_ptr->delivery_data_field_handle,
                                                                   DELIVERY_DATA_TAG,
                                                                   delivery_data,
                                                                   delivery_data_length );

         if( delivery_data_field_handle == DB_INVALID_FIELD_HANDLE ) {
            return( false );
            }

         get_address_data( address_field_handle )->delivery_data_field_handle = delivery_data_field_handle;
         return( true );
         }


      MessageStatus
      calculate_message_status( void )
      {
         MessageStatus message_status = TX_ERROR;
         MessageStatus address_status;
         DbFieldHandle db_field_handle;

         db_field_handle = message_fields.get_first_field_handle();

         while( db_field_handle != DB_INVALID_FIELD_HANDLE ) {

            switch( get_field_tag( db_field_handle ) ) {

               case TO_FIELD_TAG:
               case CC_FIELD_TAG:
                  address_status = get_address_status( db_field_handle );
                  message_status = MIN( MAX( message_status, address_status ), TX_SENT );
                  break;

               }

            db_field_handle = message_fields.get_next_field_handle( db_field_handle );
            }

         return( message_status );
         }



     /*
      * attachment fields
      */

      char *
      get_attachment_content_type( DbFieldHandle field_handle )
      {
         return( (char *) message_fields.get_field_data_ptr( field_handle ) );
         }
         
         
      char *
      get_attachment_data( DbFieldHandle field_handle,
                           int * data_length_ptr = NULL )
      {
         char * content_type = get_attachment_content_type( field_handle );
         int content_type_length = strlen( content_type ) + 1;

         if( data_length_ptr != NULL ) {
            *data_length_ptr = message_fields.get_field_data_length( field_handle ) - content_type_length;
            }

         return( content_type + content_type_length );
         }
         
         
      DbFieldHandle
      create_attachment( char * content_string,
                         char * content_string_concatenation = "",
                         char * data = "",
                         int    data_length = DB_TEXT,
                         DbFieldHandle field_handle = DB_INVALID_FIELD_HANDLE )
      {
         return( message_fields.update_field( field_handle,
                                              ATTACHMENT_FIELD_TAG,
                                              content_string, strlen( content_string ),
                                              content_string_concatenation, DB_TEXT,
                                              data, data_length ) );
         }


      bool
      add_attachment_data( DbFieldHandle field_handle,
                           char * data,
                           int    data_length = DB_TEXT )
      {
         return( message_fields.append_to_field( field_handle,
                                                 data, data_length ) );
         }

      
                   
   private:

      NetworkMessageData &
      get_data_reference( void )
      {
         return( (NetworkMessageData &) *( ( (byte *) *data_pptr ) ) );
         }


      NetworkMessageData *
      get_data_ptr( void )
      {
         return( (NetworkMessageData *)  ( ( (byte *) *data_pptr ) ) );
         }

      
      MessageAddressData *
      get_address_data( DbFieldHandle field_handle )
      {
         return( (MessageAddressData *) ( message_fields.get_field_data_ptr( field_handle ) ) );
         }

         
   };




class DefaultMessageOptions : PersistentStore
{
   public:

      MessageOptions options;

      
      DefaultMessageOptions( char * filename );
   
      
      void
      edit_options( void );
          
   };


extern DefaultMessageOptions default_message_options;



extern
bool
edit_message_options( char * title_ptr,
                      MessageOptions & options );


extern
void
edit_message_options( void );


#endif
