/****************************************************************************
 Address.h
 
 Description:
   This file defines the address book API.  It contains functions to search 
   for names in the address book, and to access the address book screen to 
   allow the user to select a name from the address book.
     
 (C) 1998-1999 Research In Motion Ltd.
 ***************************************************************************/

#ifndef ADDRESS_H
#define ADDRESS_H

#include "pager.h"

#ifdef DLLAccess
#undef DLLAccess
#endif
#ifdef _ADDRESS_BOOK_
#define DLLAccess DLLExport
#else 
#define DLLAccess DLLImport
#endif

#include "database.h"

#include "addressfieldtype.h"			// AddressFieldType defines

// AddressHandle: ABData record handle
typedef DbRecordHandle	AddressHandle;	

// AddressFieldHandle: field within ABData record
typedef DbFieldHandle	AddressFieldHandle;		

// AddressFieldType: ABData field type: FIRST_NAME, EMAIL, etc
//typedef DbFieldTag		AddressFieldType;		// defined in addressfieldtype.h

/*
 * ONESHOT_HANDLE is defined if address book handles one-shots.
 * Abscreen.h defines whether one-shots appear at top or bottom of screens
 */
#define	ONESHOT_HANDLE	(DB_NULL_HANDLE-1)

/***************************************************************
* Address Book Exported functions:
***************************************************************/
/* App -> AB User interface functions:
 *	select_name:	prompt user for name
 *	select_destination: prompt user for name & comm method
 *	edit_address_data: edit address entry
 */

/*
 * select_method:	
 *		have user select contact name and comm. method
 *		search_string is initial name prefix entered by user (may be NULl)
 * return address_handle and destination_method field handle on success
 * If specified by caller, also return on success
 *		field_data_ptr (Flash!)
 *		field_data_length
 *		field_type
 * Note: if address is One-shot:
 *		address_handle == ONESHOT_HANDLE (key)
 *		field_handle = INVALID_FIELD_HANDLE
 *		field_data = pointer to static null-terminated string buffer (may be overwritten !!!!)
 *		field_type = type of method (EMAIL, FAX, etc) - see addressfieldtype.h
 */
extern "C" DLLAccess
bool 
select_method( AddressHandle *address_handle, AddressFieldHandle *field_handle, 
			   char *search_string=NULL, const char *title=NULL, 
			   AddressFieldType *field_type=NULL,
			   char **field_data=NULL, int *field_length=NULL  );

/*
 * Select_name: display prompt as "Add to <prompt>" in action menus
 *		have user select contact name
 *		search_string is initial name prefix entered by user (may be NULl)
 *	return address_handle for given name
 */
extern "C" DLLAccess
bool 
select_name( AddressHandle *address_handle, char *search_string=NULL, 
			 const char *title=NULL );

/*
 * get_default_method
 *
 * Returns true and field handle to default omm. method for address
 *		also returns field_type if requested
 * Returns false if 
 *		invalid address handle
 *		no default comm. method for address
 */
extern "C" DLLAccess
bool 
get_default_method( AddressHandle address_handle, 
				    AddressFieldHandle *field_handle,
					AddressFieldType *field_type=NULL);


/*
 * get_address_match_count
 *
 * Returns # of addresses matching given search string
 * If 1, also returns address handle 
 */
extern "C" DLLAccess
int
get_address_match_count( const char *search_string, AddressHandle *address_handle=NULL );

/*
 * get_address_method_count
 *
 * Returns # of comm. methods for address
 * If 1, also returns comm. method field handle (default)
 */
extern "C" DLLAccess
int
get_address_method_count( AddressHandle address_handle, AddressFieldHandle *field_handle=NULL, 
						  AddressFieldType this_field_type=INVALID_FIELD_TYPE );

/*
 * get_name		get display name for given address
 *
 *	Returns false if
 *		address handle not valid
 */
extern "C" DLLAccess
bool
get_name( AddressHandle address_handle, char *buffer, int length );

/*
 * edit_address_data: edit data for address field (give that field focus)
 *  if field_handle is NULL, edit first field of type, creating field if necessary
 * 
 * returns false if
 *	Address handle is invalid
 *	field handle is !NULL and invalid
 *	field handle is NULL and field type is invalid
 */
extern "C" DLLAccess
bool
edit_address_data( AddressHandle address_handle, AddressFieldHandle field_handle=DB_INVALID_FIELD_HANDLE, 
				   AddressFieldType field_type=INVALID_FIELD_TYPE );

/*
 * delete_address_record(): remove address record from database
 * returns false if
 *      address handle is invalid
 *      record deletion failed
 * otherwise returns true
 */
extern "C" DLLAccess
bool
delete_address_record( AddressHandle address_handle );

/* App -> AB interface (no UI):
 *	find_address_handle		find address record for given data
 *	get_address_data		return data for field/type
 *	get_address_field_type	return type for field
  */
/*
 * find_address_handle: given field_type and current data, find Address record
 *
 */
extern "C" DLLAccess
AddressHandle
find_address_handle( AddressFieldType field_type, byte *data_ptr, int data_length,
					 AddressFieldHandle *field_handle=NULL );

/*
 * get_address_data: 
 *		given address and field type, return data (and length) for field
 *		if field_handle is NULL, find first of field type
 *		if field_handle is !NULL, find next of field type
 *
 * returns NULL if 
 *	address not found
 *	no fields of type, or no more fields of type
 *	
 * On success, returns data pointer and length (data may not be NULL-terminated!)
 * if length ptr not specified, app assumes field is NULL-terminated or fixed size
 */
extern "C" DLLAccess
const byte *
get_address_field_data( AddressHandle address_handle, AddressFieldHandle field_handle, 
					    int *data_length=NULL  );

/*
 * get_address_field_type: return field handle, given field_type
 *	if field_handle is NULL (or *INVALID_HANDLE), find first field of type; 
 *	else next field of same type
 */
extern "C" DLLAccess
AddressFieldHandle
get_address_field_handle( AddressHandle address_handle, AddressFieldType field_type, 
						  AddressFieldHandle *field_handle=NULL  );

/*
 * get_address_field_type: return field type, given address and field handle
 */
extern "C" DLLAccess
AddressFieldType
get_address_field_type( AddressHandle address_handle, AddressFieldHandle field_handle );


/*
 * save_address_data: save new/updated data for field within address record
 * One of the field handle or field_type must be specified (both can't be invalid)
 * updates current field for address if
 *		field_handle is valid, or 
 *		Unique is true and field type found for address
 *	creatse new field of field type if
 *		field_handle is invalid and 
 *		Unique is false or no fields of type are found 
 */
extern "C" DLLAccess
bool
save_address_data( AddressHandle &address_handle, byte *data_ptr, int data_length, 
				   AddressFieldHandle field_handle=DB_INVALID_FIELD_HANDLE, 
				   AddressFieldType field_type=INVALID_FIELD_TYPE, bool UniqueField=true,
				   bool AppendBinaryToText=false );


/*
 * add_address_field_type: update or create field type for Address Book records
 */
// Flags:
typedef unsigned short	AbFlag;
#define ABFLAG(bit)	(AbFlag)(1<<bit)

#define FIELD_TEXT			ABFLAG(0)	// 1=display field, 0=Binary field
#define FIELD_EDITABLE		ABFLAG(1)	// 1=User may edit field
#define FIELD_LOCAL			ABFLAG(2)	// 1=keep in pager, don't transmit
#define FIELD_COMM_METHOD	ABFLAG(3)	// 1=defines communication method: E-mail,Fax,....
#define FIELD_SORTABLE		ABFLAG(4)	// 1=field can be used to sort by
#define FIELD_NEW_CREATE	ABFLAG(5)	// 1=add this field type on new address create
#define FIELD_REQUIRED		ABFLAG(6)	// 1=field must exist in all address records: Name
#define FIELD_UNIQUE		ABFLAG(7)	// 1=field data is unique in address book: Name ?
#define FIELD_MULTIPLE		ABFLAG(8)	// 1=multiple fields of this type per address (eg. Email)
#define FIELD_DEFAULT		(FIELD_TEXT|FIELD_EDITABLE)

extern "C" DLLAccess
bool 
add_address_field_type( const char *field_name, AddressFieldType field_type, 
					    AbFlag flags=FIELD_DEFAULT,
						bool update_flags_only=false );

extern "C" DLLAccess
const char *get_field_type_name( AddressFieldType field_type, int offset=0 );


#endif	// ifndef ADDRESS_H
