/****************************************************************************
 Choice.h
 
 Description:
    Choice boxes are effectively one-line text boxes that always 
    have a label. Choice boxes right justify the data, which is either a 
    range of numeric data or an array of strings.  The user can scroll
    between the choices and select the appropriate one.
 
 Example:
    ...
    Choice myChoice;
    myChoice.SetLabel( "Here are the choices:" );
    myChoice.SetChoices( 1, MAX_CHOICES, 12 );
    screen.AddField( myChoice );
    ...

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef choice_h_included
#define choice_h_included
#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
#include "field.h"

class DLLAccess Choice : public Field
{
    public: // used by client

        Choice(); 
        Choice( char const * const pnewLabel,
                char const * const * const newStringArray,
                int const newIndex=0,
                int const Justify=LCD_RIGHT_JUSTIFIED );
        
        Choice( char const * const pnewLabel,
                unsigned int const newStartValue,
                unsigned int const newEndValue,
                int const newIndex=0,
                int const Increment=1,
                int const Justify=LCD_RIGHT_JUSTIFIED );

        void SetChoices( char const * const * const newStringArray, int const newIndex=0 );
        void SetChoices( unsigned int const newStartValue, 
                         unsigned int const newEndValue, int const newIndex=0, 
                         int const Increment=1 );

        int GetSelectedIndex() const;  
        void SetSelectedIndex( int const newIndex ); 
        
        void SetLabel( char const * const pnewLabel );
        char const * const GetLabel() {return (pLabel);}
        int GetSelectedValue();

        void SetJustification( int const Justify=LCD_LEFT_JUSTIFIED );

        RESULT ChangeChoice( UIEngine &newCallingUI, 
                             void (( *CallbackFunction )(int Value)) = NULL );



        FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );
        void SetNumEntries( int const NumEntries );

    public: //base class functionality
        virtual void Redraw();
        void ResetDimensionDependentData(); //used for updating display parms on the fly
        void Draw( int const PixelOnScreen, FieldLocation const AnchoredFrom, int &TopPixel, 
                   int &HeightinPixels, 
                   bool &MoreDataAtTop, bool &MoreDataAtBottom );
										// null for toppixel and Height means the caller 
        void UnDraw();
        bool GiveFocus( int const DirectionMagnitude );
        void TakeFocus();
        int QueryPixelHeight( FieldLocation const StartFrom, int const MaximumRelevant );
        int QueryHeightInArea( FieldLocation const StartFrom );
        int QueryLineHeight( int const MaximumRelevant );
        bool IsFocusVisible(); 
        RESULT HandleInput( MESSAGE const &InputMessage );
		virtual void NowDisplaying( int const newIndex );
        void PutData( const char * const newData );

    protected:

        bool FocusFlag;
        bool OnLCDFlag;
        bool UseString;
    
        char const * pLabel;
        char const * pData;
        int WidthofLabel;
        int Index; 
        int MaxIndex;
        int Increment;
        char NumberString[12];

        // either String Array or Start and Stop
        char const *const * StringArray; //null or non null
        unsigned int Start;
        unsigned int End;


        
    protected: // implementation functions

        void DrawChoice ( bool * pMoreDataAtTop = NULL, bool * pMoreDataAtBottom = NULL );
        void Initialize();

        int NextNumber( int const FirstDigit, int const CurrentValue );

};


class DLLAccess BeepChoice : public Choice
{
   public: // used by client

      BeepChoice();
      BeepChoice( char * Label, int CurrentChoice );

      RESULT HandleInput( MESSAGE const &InputMessage );
      int GetBeep( void );
      void SetBeep( int newBeepNumber );
      FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );

};

class DLLAccess YesNoChoice : public Choice
{
   public: // used by client

      YesNoChoice( char * Label, bool CurrentChoice );

      bool GetFlag( void );
      void SetFlag( bool newFlagValue );
      FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );

};

class DLLAccess TopBottomChoice : public Choice
{
   public: // used by client

      TopBottomChoice( char * Label, bool CurrentChoice );
      bool GetFlag( void ); 
      void SetFlag( bool newFlagValue );
      FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );
};

#endif // choice_h_included
