/****************************************************************************
 Database.h
 
 Description:
     A full-featured database application that stores C++ objects of a special
     form.  Fast sorting and indexing methods are built in.

 Class Diagram:
                                                       

                        +-------------+     +----------+     +--------+           +--------+
                        |  Database   |*   1|          |1   *|        |1         1| Flash  |
                        |    View     |-----| Database |-----| Flash  |- - - - - -| Data   |
                        |             |     |          |     |        |           |        |
                        +-------------+     +----------+     +--------+           +--------+
                           ^   ^   ^                           ^    ^                 ^
                           |   |   |                           |    |                 |
    +--------+    +----------+ | +----------+          +--------+  +--------+     +--------+
    |Database|*  1| Database | | | Database |          |Database|  | Flash  |     | Flash  |
    | Index  |----|  Index   | | |   List   |          | Record |  | Object |- - -| Object |
    | Search |    |   View   | | |   View   |          |        |  |        |     |  Data  |
    +--------+    +----------+ | +----------+          +--------+  +--------+     +--------+
                           ^   |   ^   ^                   ^           ^              ^
                           |   |   |   |                   |           |              |
                           |   |   | +----------+      +--------+  +--------+     +--------+
                           |   |   | | Database |      |  User  |  |  User  |     |  User  |
                           |   |   | |  Sorted  |      |Database|  | Flash  |- - -| Flash  |
                           |   |   | |   List   |      | Record |  | Object |     | Object |
                           |   |   | |   View   |      |        |  |  Data  |     |        | 
                           |   |   | +----------+      +--------+  +--------+     +--------+
                           |   |   |   ^                             
                           |   |   |   |                             
                 +--------------------------+
                 |           User           |
                 |         Database         |
                 |           View           |     
                 +--------------------------+
 

 Details:
    A FlashObject is a C++ object whose member data may be stored in Flash or
    RAM memory. The member data is contained in a FlashObjectData structure.
    A FlashObject may have virtual functions. 

    There should be a one to one correspondence between FlashObject objects
    and FlashObjectData structures.

    A Database contains many Flash objects.

    A Database may have several DatabaseViews associated with it. A DatabaseView
    generally contains a list of DbRecordHandles that correspond to Flash objects
    that belong in the view. For example, the message list database may have 
    inbox and outbox views.

    The DatabaseListView simply keeps all handles associated with the view in
    an dynamic array.

    The DatabaseSortedListView keeps all handles associated with the with the
    view sorted in an array. For example, a database of names could keep one
    view sorted by first name, and another view sorted by last name. 

    The DatabaseIndexView allows the application to build an index of the
    database objects. This index may be searched using a DatabaseIndexSearch
    object.

    The DatabaseIndexSearch allows the application to quickly search
    through the database index and find those entries that contain a given prefix.
    It is expected that AutoText and address book will use this facility.


 Copyright (C) 1998-1999 Research In Motion Limited
****************************************************************************/

#ifndef DATABASE_H
#define DATABASE_H

#include "warning.h"
#include <limits.h>

#include "type.h"
#include "utilities.h"

#include "pager.h"
#include "filesys.h"

#include "bitfield.h"
#include "darray.h"


/* 
 * typedefs and constants
 */
typedef HandleType DbDatabaseHandle;
typedef HandleType DbRecordHandle;

const HandleType DB_NULL_HANDLE = -1;
const HandleType DB_MAX_NUM_HANDLES = NUM_HANDLE;
 


/*************************************************************************
 *
 * Database Flash Object Data Struct
 *
 * The FlashData structure contains all member data associated with 
 * the object.
 *
 *************************************************************************/

struct FlashData 
{
   };
   
   
class Database;


class Flash
{

   protected:
   
      FlashData ** data_pptr;
      
      FlashData *  ram_data_ptr;
      bool loaded_flag;
      
      const Database & database;
      DbRecordHandle db_record_handle;
      FlashData * original_flash_ptr;
      
     
      Flash( const Database & init_database,
             DbRecordHandle init_db_record_handle = DB_NULL_HANDLE,
             FlashData * init_ram_data_ptr = NULL );
         
        
   public:
      
      virtual
      ~Flash()
      {
         }
         
      
      virtual
      bool
      reconstruct( DbRecordHandle init_db_record_handle );
         
         
      FlashData ***
      get_data_ppptr( void )
      {
         return( &data_pptr );
         }

      virtual
      bool
      load( void ) 
      {
         return( false );
         }
      
      
      virtual
      DbRecordHandle
      save( void ) 
      {
         return( DB_NULL_HANDLE );
         }
      

      void
      zero_data( void );


      virtual
      void
      delete_object( bool zero_flag = false );


      DbRecordHandle
      get_db_record_handle( void )
      {
         return( db_record_handle );
         }


      virtual
      bool
      clear_db_record_handle( void )
      {
         if( load() == false ) {
            return( false );
            }

         db_record_handle = DB_NULL_HANDLE;
         return( true );
         }


      virtual
      bool
      moved_in_flash( void )
      {
         return( !loaded_flag && original_flash_ptr != *data_pptr );
         }




   protected:
   
      bool
      load_data( int data_length );
         
         
      DbRecordHandle
      save_data( int data_length );
      
     
   };         
         
         
/*************************************************************************
 * 
 * FlashObject class
 *
 *************************************************************************/
       
struct FlashObjectData : public FlashData 
{
   };
   
   
class FlashObject : public Flash
{
   protected:
   
      FlashObject( Database & init_database,
                   DbRecordHandle init_db_record_handle = DB_NULL_HANDLE,
                   FlashData * init_ram_data_ptr = NULL )
         : Flash( init_database, init_db_record_handle, init_ram_data_ptr )
      {
         }
         
         
   public:
      
   };
         
      
         
        
/*************************************************************************
 * 
 * DatabaseRecord class
 *
 *************************************************************************/
         
typedef byte DbFieldHandle;
typedef byte DbFieldTag;
   
const DbFieldHandle  DB_INVALID_FIELD_HANDLE = ( (DbFieldHandle)  -1 );
const DbFieldTag     DB_INVALID_FIELD_TAG    = ( (DbFieldTag)     -1 );
const unsigned short DB_INVALID_DATA_LENGTH  = ( (unsigned short) -1 );
      
const int DB_TEXT = -INT_MAX;




      
class DatabaseRecord : public Flash
{
   private:
   
      struct DbField {

         DbFieldHandle field_handle;
         DbFieldTag field_tag;
         unsigned short data_length;
         byte data[0];

         };


      class DbFieldHandleCache 
      {
         private:

            DbFieldHandle field_handle;
            int offset;
   

         public:

            DbFieldHandleCache( void )
            {
               reset();
               }


            void
            reset( void )
            {
               field_handle = DB_INVALID_FIELD_HANDLE;
               }


            void
            set( DbFieldHandle new_field_handle,
                 DbField *     new_field_ptr,
                 DbField *     first_field_ptr )
            {
               field_handle = new_field_handle;
               offset       = (byte *) new_field_ptr - (byte *) first_field_ptr;
               }


            bool
            check( DbFieldHandle const new_field_handle,
                   bool loaded_flag )
            {
               return(    field_handle == new_field_handle
                       && field_handle != DB_INVALID_FIELD_HANDLE );
               }


            DbField *
            get( DbField * first_field_ptr )
            {
               return( (DbField *) ( (byte *) first_field_ptr + offset ) );
               }

         };

      DbFieldHandleCache field_handle_cache;

   
   public:
   
      DatabaseRecord( const Database & init_database );
         
        
      DatabaseRecord( const Database & init_database,
                      DbRecordHandle init_db_record_handle );
         
        
      ~DatabaseRecord();
         

      bool
      initialize_record( void );

      
      virtual
      bool
      reconstruct( DbRecordHandle init_db_record_handle );


      virtual
      bool
      load( void );
      
      
      virtual
      DbRecordHandle
      save( void )
      {
         return( save_data( get_record_data_length() ) );
         }
      
      
      bool
      error( void ) const
      {
         return( loaded_flag && ram_data_ptr == NULL ? true : false );
         }
         
         
      DbFieldHandle
      get_first_field_handle( void )
      {
         DbField * field_ptr = get_first_field_ptr();
         DbFieldHandle field_handle = field_ptr->field_handle;

         field_handle_cache.set( field_handle, field_ptr, get_first_field_ptr() );
         
         return( field_handle );
         }
      
      
      DbFieldHandle
      get_first_field_handle( DbFieldTag field_tag )
      {
         DbField * field_ptr = get_first_field_ptr( field_tag );
         DbFieldHandle field_handle = field_ptr->field_handle;

         field_handle_cache.set( field_handle, field_ptr, get_first_field_ptr() );

         return( field_handle );
         }
      
      
      DbFieldHandle
      get_next_field_handle( DbFieldHandle field_handle )
      {
         DbField * field_ptr = get_next_field_ptr( find_field_ptr( field_handle ) );
         field_handle = field_ptr->field_handle;
         
         field_handle_cache.set( field_handle, field_ptr, get_first_field_ptr() );

         return( field_handle );
         }
   
      
      DbFieldHandle
      get_next_field_handle( DbFieldHandle field_handle,
                             DbFieldTag    field_tag )
      {
         DbField * field_ptr = get_next_field_ptr( find_field_ptr( field_handle ),
                                                   field_tag );
         field_handle = field_ptr->field_handle;

         field_handle_cache.set( field_handle, field_ptr, get_first_field_ptr() );

         return( field_handle );
         }
   
      
      void * const
      get_field_data_ptr( DbFieldHandle field_handle )
      {
         DbField * field_ptr = find_field_ptr( field_handle );
         if( field_ptr->field_handle == DB_INVALID_FIELD_HANDLE ) {
            return( NULL );
            }
         return( &field_ptr->data[0] );
         }
      
      
      int
      get_field_data_length( DbFieldHandle field_handle )
      {
         DbField * field_ptr = find_field_ptr( field_handle );
         return( field_ptr->data_length );
         }
      
      
      DbFieldTag
      get_field_tag( DbFieldHandle field_handle )
      {
         DbField * field_ptr = find_field_ptr( field_handle );
         return( field_ptr->field_tag );
         }
      
      
      bool
      set_field_tag( DbFieldHandle field_handle,
                     DbFieldTag field_tag );
      
      
      DbFieldHandle
      add_field( DbFieldTag   field_tag,
                 int          data_length = 0 )        
      {
         return( update_field( DB_INVALID_FIELD_HANDLE, field_tag,
                               NULL, data_length,
                               NULL, 0,
                               NULL, 0 ) );
         }
                    
                   
      DbFieldHandle
      add_field( DbFieldTag   field_tag,
                 const void * data1_ptr,    
                 const void * data2_ptr    = NULL,
                 const void * data3_ptr    = NULL,
                 int          data1_length = DB_TEXT,          // DB_TEXT for text
                 int          data2_length = DB_TEXT,          // DB_TEXT for text
                 int          data3_length = DB_TEXT )         // DB_TEXT for text
      {
         return( update_field( DB_INVALID_FIELD_HANDLE, field_tag,
                               data1_ptr, data1_length,
                               data2_ptr, data2_length,
                               data3_ptr, data3_length ) );
         }
                    
                   
      DbFieldHandle
      add_field( DbFieldTag   field_tag,
                 const void * data1_ptr,    
                 int          data1_length,                    // DB_TEXT for text
                 const void * data2_ptr    = NULL,
                 int          data2_length = DB_TEXT,          // DB_TEXT for text
                 const void * data3_ptr    = NULL,
                 int          data3_length = DB_TEXT )         // DB_TEXT for text
      {
         return( update_field( DB_INVALID_FIELD_HANDLE, field_tag,
                               data1_ptr, data1_length,
                               data2_ptr, data2_length,
                               data3_ptr, data3_length ) );
         }

      
      DbFieldHandle
      update_field( DbFieldHandle field_handle,
                    const void *  data1_ptr,    
                    const void *  data2_ptr    = NULL,
                    const void *  data3_ptr    = NULL,
                    int           data1_length = DB_TEXT,          // DB_TEXT for text
                    int           data2_length = DB_TEXT,          // DB_TEXT for text
                    int           data3_length = DB_TEXT )         // DB_TEXT for text
      {
         return( update_field( field_handle, DB_INVALID_FIELD_TAG, 
                               data1_ptr, data1_length, 
                               data2_ptr, data2_length, 
                               data3_ptr, data3_length ) );
         }

      
      DbFieldHandle
      update_field( DbFieldHandle field_handle,
                    const void *  data1_ptr,
                    int           data1_length,                    // DB_TEXT for text
                    const void *  data2_ptr    = NULL,
                    int           data2_length = DB_TEXT,          // DB_TEXT for text
                    const void *  data3_ptr    = NULL,
                    int           data3_length = DB_TEXT )         // DB_TEXT for text
      {
         return( update_field( field_handle, DB_INVALID_FIELD_TAG, 
                               data1_ptr, data1_length, 
                               data2_ptr, data2_length, 
                               data3_ptr, data3_length ) );
         }

      
      DbFieldHandle
      update_field( DbFieldHandle field_handle,
                    DbFieldTag    field_tag,
                    const void *  data1_ptr,
                    const void *  data2_ptr    = NULL,
                    const void *  data3_ptr    = NULL,
                    int           data1_length = DB_TEXT,          // DB_TEXT for text
                    int           data2_length = DB_TEXT,          // DB_TEXT for text
                    int           data3_length = DB_TEXT )         // DB_TEXT for text
      {
         return( update_field( field_handle, field_tag, 
                               data1_ptr, data1_length, 
                               data2_ptr, data2_length, 
                               data3_ptr, data3_length ) );
         }

      
      DbFieldHandle
      update_field( DbFieldHandle field_handle,
                    DbFieldTag    field_tag,
                    const void *  data1_ptr,
                    int           data1_length,                    // DB_TEXT for text
                    const void *  data2_ptr    = NULL,
                    int           data2_length = DB_TEXT,          // DB_TEXT for text
                    const void *  data3_ptr    = NULL,
                    int           data3_length = DB_TEXT );        // DB_TEXT for text


      bool
      prepend_to_field( DbFieldHandle field_handle,
                        const void *  data_ptr,
                        int           data_length = DB_TEXT );     // DB_TEXT for text

      
      bool
      append_to_field( DbFieldHandle field_handle,
                       const void *  data_ptr,
                       int           data_length = DB_TEXT );      // DB_TEXT for text

      
      bool
      replace_field_data( DbFieldHandle field_handle,
                          int           offset,
                          const void *  data_ptr,
                          int           data_length = DB_TEXT );   // DB_TEXT for text
                
      
      void
      delete_field( DbFieldHandle field_handle );
      
      
      void
      delete_fields( DbFieldTag field_tag );
      
      
      int
      get_record_data_length( void ) const
      {
         return( get_data_length( get_first_field_ptr() ) );
         }
      
        
         
   protected:
   
      bool
      realloc_data_ptr( int new_data_length );
     
         
      DbField *
      get_first_field_ptr( void ) const
      {
         return( (DbField *) *data_pptr );
         }
         
         
      DbField *
      get_first_field_ptr( DbFieldTag field_tag ) const;
   
   
      DbField *
      get_next_field_ptr( DbField * field_ptr ) const
      {
         return( (DbField *) ( (byte *) field_ptr + sizeof( DbField ) + field_ptr->data_length ) );
         }
      
     
      DbField *
      get_next_field_ptr( DbField * field_ptr,
                          DbFieldTag field_tag ) const;
   
  
      DbField *
      find_field_ptr( DbFieldHandle field_handle );
      
     
      DbFieldHandle
      get_new_field_handle( void );
          
     
      void
      delete_field( DbField * field_ptr );
      
      
      int
      get_data_length( DbField * field_ptr ) const;
      

      DbField *
      insert_field( DbFieldTag field_tag,
                    int data_length = 0 );


   };
      
      
      
      
   
  
         
         
         
         
      
      
/*************************************************************************
 *
 * Database class
 *
 *************************************************************************/


class DatabaseView;


class Database {
   
   protected:
   
      char * database_name;
      DbDatabaseHandle db_database_handle;
      
      enum { MAX_NUM_VIEWS = 16 };
      
      int num_views;
      DatabaseView * view_ptrs[ MAX_NUM_VIEWS ];
      
   
   public:
   
      enum Action { ADD, UPDATE, DELETE };
      
      
      Database( char * init_name );


      int
      get_num_records( void ) const;


      virtual
      bool
      delete_record( DbRecordHandle db_record_handle ) const;
      
 
      virtual
      void
      delete_all_records( bool zero_flag );


     /* 
      * Flash API
      */
      DbRecordHandle
      save_record( DbRecordHandle db_record_handle,
                   void * data_ptr,
                   int data_length ) const;


      
     /*
      * DatabaseView API
      */
      bool
      register_view( DatabaseView * view_ptr );
      
     
      void
      deregister_view( DatabaseView * view_ptr );
      
     
      bool
      reload_view( DatabaseView * view_ptr ) const;
      

      DbRecordHandle
      get_first_record( void ) const
      {
         DbRecordHandle db_record_handle = DbFirstRec( db_database_handle );
         return( db_record_handle < 0 ? DB_NULL_HANDLE : db_record_handle );
         }


      DbRecordHandle
      get_next_record( DbRecordHandle db_record_handle ) const
      {
         db_record_handle = DbNextRec( db_record_handle );
         return( db_record_handle < 0 ? DB_NULL_HANDLE : db_record_handle );
         }


      void
      save_views( DatabaseView * current_view = NULL ) const;


   private:

      bool
      update_view( DatabaseView * view_ptr,
                   Action action,
                   DbRecordHandle db_record_handle ) const;

                   
      bool
      update_views( Action action,
                    DbRecordHandle db_record_handle ) const;
        
      
   };
      
      
      
     
/*************************************************************************
 *
 * Database View Class
 *
 *************************************************************************/
 
/*
 * 
 * A DatabaseView provides a very simple way to monitor changes to a database.
 * 
 * Once a DatabaseView is registered with a database, the database will call 
 * process_database_update() for each change to the database.
 * 
 */
   
class DatabaseView {
   
   protected:
      
      Database & database;
      
     
      DatabaseView( Database & init_database )
         : database( init_database )
      {
         database.register_view( this );
         }


      DatabaseView( DatabaseView & view )
         : database( view.database )
      {
         database.register_view( this );
         }
         
         
   public:
   
      virtual
      ~DatabaseView()
      {
         database.deregister_view( this );
         }
         
         
     /*
      * process_database_update() will be called for every change
      * made to the database (i.e. additions, updates, and deletions).
      */
      virtual
      bool
      process_database_update( Database::Action action,
                               DbRecordHandle db_record_handle );
         
         
     /*
      * is_view_member() will be called to check if the flash object
      * belongs in this view.
      */
      virtual
      bool
      is_view_member( DbRecordHandle db_record_handle )
      {
         return( true );
         }
         
         
     /*
      * insert() is called to insert a new entry into the view
      */
      virtual
      bool
      insert( DbRecordHandle db_record_handle )
      {
         return( true );
         }
              
              
     /*
      * update() is called when a view entry has changed
      */
      virtual
      bool
      update( DbRecordHandle db_record_handle )
      {
         return( true );
         }
              
             
     /*
      * delete_handle() is called when an entry is to be deleted from a view
      */
      virtual
      bool
      delete_handle( DbRecordHandle db_record_handle )
      {
         return( true );
         }
      
     
     /*
      * reload_view() will cause the database to call 
      * process_database_update() for each Flash object. This call
      * could be used to reinitialize a view.
      */
      virtual
      bool
      reload_view( void )
      {
         reset_view();
         return( database.reload_view( this ) );
         }


     /*
      * reset_view() will reinitialize the view to an empty view
      */
      virtual
      void
      reset_view( void )
      {
         }


     /*
      * save_view() should, if possible, free any allocated memory
      */
      virtual
      bool
      save( void )
      {
         return( true );
         }
         
         
   };
   
   
   
/*************************************************************************
 * 
 * A DatabaseListView provides a very simple way to remember which 
 * FlashObjects are in the database. 
 * 
 * This view keeps a dynamically sized array of handles in RAM.
 *
 *************************************************************************/

class DatabaseListView : public DatabaseView
{
   private:
         
      DynamicArray< DbRecordHandle > handle_array;

   
   protected:
      
      DatabaseListView( Database & init_database,
                        int init_num_entries = 10,
                        bool use_flash_flag = false,
                        char * filename = NULL )
         : DatabaseView( init_database )
         , handle_array( DB_NULL_HANDLE, init_num_entries, use_flash_flag, filename )
      {
         }


      DatabaseListView( DatabaseListView & view,
                        bool use_flash_flag = false,
                        char * filename = NULL )
         : DatabaseView( view )
         , handle_array( DB_NULL_HANDLE, view.get_num_handles(), use_flash_flag, filename )
      {
         copy_view( view );
         }
         
        
   public:
         
      virtual
      ~DatabaseListView()
      {
         }
         

      bool
      load( void )
      {
         return( handle_array.load_array() );
         }


      virtual
      bool
      save( void )
      {
         return( handle_array.save_array( true ) );
         }


      virtual
      void
      reset_view( void )
      {
         handle_array.reset();
         }


      int
      get_num_handles( void ) const
      {
         return( handle_array.get_num_entries() );
         }
         
         
      int
      find_index( DbRecordHandle db_record_handle ) const
      {
         return( handle_array.find_entry( db_record_handle ) );
         }
      
      
      virtual
      bool
      insert( DbRecordHandle db_record_handle )
      {
         return( insert_at( get_num_handles(), db_record_handle ) );
         }
              
   
      virtual
      bool
      insert_at( int index, 
                 DbRecordHandle db_record_handle )
      {
         return( handle_array.insert( index, db_record_handle ) );
         }
   
   
      virtual
      bool
      delete_handle( DbRecordHandle db_record_handle )
      {
         return( delete_index( find_index( db_record_handle ) ) );
         }

         
      bool
      delete_index( int index )
      {
         return( handle_array.delete_index( index ) );
         }
         
         
      DbRecordHandle &
      operator[]( int index ) const
      {
         return( handle_array[ index ] );
         }
         
        
      DbRecordHandle
      get_handle( int index ) const
      {
         return( handle_array[ index ] );
         }
         
        
      bool
      copy_view( DatabaseListView & copy_view )
      {
         return( handle_array.copy_array( copy_view.handle_array ) );
         }


      bool
      restrict_view( void );


      bool
      copy_and_restrict_view( DatabaseListView & copy_view );


      int
      get_cursor( void ) const
      {
         return( handle_array.get_cursor() );
         }
         
         
      void
      set_cursor( int new_cursor )
      {
         handle_array.set_cursor( new_cursor );
         }


      void
      shrink( void )
      {
         handle_array.shrink_array();
         }


     /*
      * private functions
      */

      DbRecordHandle *
      get_array_ptr( void )
      {
         return( handle_array.get_array_ptr() );
         }

      
      bool
      grow_array( int index )
      {
         return( handle_array.grow_array( index ) );
         }

      
      void
      set_num_entries( int num_entries )
      {
         handle_array.set_num_entries( num_entries);
         }
      
  };
   
   
   
/*
 * A DatabaseSortedListView provides a very simple way to remember which 
 * FlashObjects are in the database. 
 * 
 * This view keeps a dynamically sized array of handles in RAM.
 *
 * The array is sorted according to compare_objects().
 */
  
class DatabaseSortedListView : public DatabaseListView
{
   
   protected:
   
      DatabaseSortedListView( Database & init_database,
                              int init_num_entries = 10,
                              bool use_flash_flag = false )
         : DatabaseListView( init_database, init_num_entries, use_flash_flag )
      {
         }
         
         
      DatabaseSortedListView( DatabaseListView & view, 
                              bool use_flash_flag = false )
         : DatabaseListView( view, use_flash_flag )
      {
         }
         
        
   public:
      
      virtual
      ~DatabaseSortedListView()
      {
         }
         
         
      virtual
      bool
      insert( DbRecordHandle db_record_handle );
              
              
      virtual
      bool
      update( DbRecordHandle db_record_handle )
      {
         return( resort( db_record_handle) );
         }
         
         
      bool
      resort( DbRecordHandle db_record_handle );
              
              
      virtual
      int
      compare_objects( DbRecordHandle db_record_handle1,
                       DbRecordHandle db_record_handle2 )
      {
        /* 
         * This function is called to determine if
         * db_record_handle1 is to appear before ( < 0 ) 
         * or after ( > 0 ) db_record_handle2
         */
         
         return( 0 );
         }
   
   
         
   };
   
   
   
  
/*************************************************************************
 * 
 * A DatabaseIndexView provides a very simple way to find  
 * FlashObjects in the database. 
 * 
 *************************************************************************/

class DatabaseIndexSearch;

class DbRecordBitField : public BitField
{
   private:

      unsigned int bits[ BITFIELD_NUM_INTS( DB_MAX_NUM_HANDLES ) ];
      

   public:

      DbRecordBitField( void )
         : BitField( DB_MAX_NUM_HANDLES, bits, sizeof( bits ) )
      {
         }

   };



class DatabaseIndexView : public DatabaseView 
{
   private:

      enum { NUM_CHARACTERS = 1 + 26 };  

      DynamicArray< unsigned int > prefix_table[ NUM_CHARACTERS ];

      int save_choice;

      
   protected:
   
      DatabaseIndexView( Database & init_database,
                         int init_num_entries = 10,
                         bool use_flash_flag = true,
                         char * filename = NULL );
         
         
      
   public:

      virtual
      ~DatabaseIndexView()
      {
         }

      
      virtual
      void
      reset_view( void );


      bool
      error( void );

      
      bool
      insert_words( DbRecordHandle db_record_handle,
                    const char * string,
                    int num_words = INT_MAX,           // negative for string_length
                    bool first_word_flag = false );  
      
      bool
      update( DbRecordHandle db_record_handle )
      {
         if( !delete_handle( db_record_handle ) ) {
            return( false );
            }
         return( insert( db_record_handle ) );
         }


      bool
      delete_handle( DbRecordHandle db_record_handle );

/*
      void
      build_handle_list( DatabaseIndexSearch & search_view, 
                         const char * string,
                         int string_length = INT_MAX,
                         DbRecordBitField * first_word_handle_list_ptr = NULL );
  */       
      bool
      build_handle_list( DatabaseIndexSearch & search_view, 
                         const char * string,
                         int num_words = INT_MAX,         // negative for string_length
                         byte * hit_count = NULL );
         

      virtual
      bool
      save( void );


   private:

      bool
      reset_prefix_array( int i );

         
   };
   
   
   
/*************************************************************************
 * 
 * A DatabaseIndexSearch provides a very simple way to search the
 * Database index.
 * 
 *************************************************************************/

class DatabaseIndexSearch 
{
   protected:
   
      DatabaseIndexView & index;
   
      DbRecordBitField handle_list;
      
      
   public:

      DatabaseIndexSearch( DatabaseIndexView & init_index )
         : index( init_index )
      {
         }
 
         
      virtual
      ~DatabaseIndexSearch()
      {
         }
         
         
      void
      reset( void )
      {
         handle_list.set_all();
         }
      
      
      int
      get_match_count( void )
      {
         return( handle_list.count_set_bits() );
         }
      
     
      int
      get_match_count( const char * string,
                       int num_words = INT_MAX )         // negative for string_length
      {
         if( string ) {
            index.build_handle_list( *this, string, num_words );
            }
            
         return( handle_list.count_set_bits() );
         }
      
     
      bool
      build_view( DatabaseView & view,
                  const char * string = NULL,
                  int num_words = INT_MAX,                // negative for string_length
                  bool assume_view_member_flag = true );


      bool
      copy_view( DatabaseListView & master_view,
                 DatabaseListView & copy_view,
                 const char * string = NULL,
                 int num_words = INT_MAX,                // negative for string_length
                 bool assume_view_member_flag = true );

                 
      bool
      copy_view_keeping_word_order( DatabaseListView & master_view,
                                    DatabaseListView & copy_view,
                                    const char * string, 
                                    int num_words = INT_MAX,            // negative for string_length
                                    bool assume_view_member_flag = true );
                   
                                    
      bool
      restrict_view( DatabaseListView & view,
                     const char * string,
                     int num_words = INT_MAX,             // negative for string_length
                     bool assume_view_member_flag = true );


      int
      count_matching_handles( DatabaseListView & view,
                              const char * string = NULL,
                              int num_words = INT_MAX,    // negative for string_length
                              bool assume_view_member_flag = true );

     
      bool
      is_matching_handle( DbRecordHandle db_record_handle ) const
      {
         return( handle_list.is_set( db_record_handle ) );
         }
         
         
      DbRecordHandle
      get_first_matching_handle( void )
      {
         return( (DbRecordHandle) handle_list.get_first_set_bit() );
         }
         
        
      DbRecordHandle
      get_next_matching_handle( DbRecordHandle db_record_handle )
      {
         return( (DbRecordHandle) handle_list.get_next_set_bit( db_record_handle ) );
         }
         
          
      void
      operator = ( DatabaseIndexSearch & rhs_view )
      {
         handle_list = rhs_view.handle_list;
         } 
         
         
      void
      operator &= ( DatabaseIndexSearch & rhs_view )
      {
         handle_list &= rhs_view.handle_list;
         } 
         
        
      void
      operator |= ( DatabaseIndexSearch & rhs_view )
      {
         handle_list |= rhs_view.handle_list;
         } 
         
        
      void
      operator ^= ( DatabaseIndexSearch & rhs_view )
      {
         handle_list ^= rhs_view.handle_list;
         } 
         
         
      void
      operator = ( DbRecordBitField & rhs_handle_list )
      {
         handle_list = rhs_handle_list;
         } 
         
         
      void
      operator &= ( DbRecordBitField & rhs_handle_list )
      {
         handle_list &= rhs_handle_list;
         } 
         
        
      void
      operator |= ( DbRecordBitField & rhs_handle_list )
      {
         handle_list |= rhs_handle_list;
         } 
         
        
      void
      operator ^= ( DbRecordBitField & rhs_handle_list )
      {
         handle_list ^= rhs_handle_list;
         } 
         
         
   };
   
   
   
     
#endif
