/****************************************************************************
 DateTime.h
 
 Description:
    This class defines two classes: one for dates and one for times.  In either
    case the user can modify the date or time using the roller wheel or the
    keyboard.
     
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef datetime_h_included
#define datetime_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
#include "field.h"
#include "choice.h"
#include "dialog.h"

const int DATETIME_BUFFER_SIZE = 20;

typedef enum 
{
    MMM_DD_YYYY,            // eg. January 1, 1990
    YYYY_MM_DD,             // eg. 1990/01/01
    DEF_DATE_FORMAT=MMM_DD_YYYY
} DateFormat;

class DLLAccess EditDate : public Field
{
    public: // used by client
    
		EditDate();
		EditDate( char const * const pnewLabel, 
                  TIME const &datetime,
                  DateFormat const format = DEF_DATE_FORMAT );
		
        RESULT  ChangeChoice( UIEngine &newCallingUI, void (*func)(int Value) );

        void    UpdateDate( int const newChoiceIndex );
        void    SetDate( TIME const &datetime );
                                  
		void	SetLabel( char const * const pnewLabel );

		void	SetDateFormat( DateFormat const format );
    
		TIME	GetDate() 
        { 
            VerifyDate();
            return DateTime; 
        }

		char* 	GetDateString( char * const buffer = NULL, int const length = 0 );	// returns date buffer and/or length
		void	SetJustification( int const justify = LCD_LEFT_JUSTIFIED );

        virtual 
        void    Redraw();


	public: // used by UI Engine
	//Ancestor functionality
        void      ResetDimensionDependentData(); //used for updating display parms on the fly
        void      Draw( int const PixelOnScreen, FieldLocation const AnchoredFrom, 
                        int &TopPixel, int &HeightinPixels, 
                        bool &MoreDataAtTop, bool &MoreDataAtBottom );
		void      UnDraw();
		bool      GiveFocus( int const DirectionMagnitude );
		void      TakeFocus();
		int       QueryPixelHeight( FieldLocation const StartFrom, int const MaximumRelevant );
        int       QueryHeightInArea( FieldLocation const StartFrom );
        int       QueryLineHeight( int const MaximumRelevant );
		bool      IsFocusVisible(); 

        RESULT    HandleInput( MESSAGE const &InputMessage );
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );
    

	protected:   
  
		RESULT HandleCharacter( MESSAGE const &InputMessage );

        void   DrawDate( bool * pMoreDataAtTop = NULL, bool * pMoreDataAtBottom = NULL );

		void   ChangeDate( int const DirectionMagnitude );
		void   GetFieldOffset( int StartAtPixel, int *Offset, int *EndOffset );
		bool   PrevField();	// select previous subfield, if any
		bool   NextField();	// select next subfield, if any

        void   VerifyDate(); //checks to make sure the values entered are OK

        bool   FocusFlag;			// true when field has focus
        bool   OnLCDFlag;			// true when field is on screen
        
        unsigned char  lastGoodDate;
        unsigned char  lastGoodMonth;
        unsigned short lastGoodYear;
        
		char const * pLabel;
		int    WidthofLabel;		// width of label in pixels
   
		typedef enum 
        {
			FIELD_MONTH,
			FIELD_DAY,
            FIELD_YEAR
		} DateField;
		
        DateField Index;		// index into which subfield has current focus

		char DateBuffer[DATETIME_BUFFER_SIZE];	// current date string

		TIME	    DateTime;		// current date/time
		DateFormat	Format;     // format 
		int			Justify;
};

typedef enum 
{
	HOUR_12,			// eg. 2:34 PM
	HOUR_24,			// eg. 14:34
	HOUR_12_SH,			// eg. 2:34p
	DEF_TIME_FORMAT=HOUR_12
} TimeFormat;

typedef char TimeZone;
const TimeZone GMT=0;

class DLLAccess EditTime : public Field
{
	
	public: // used by client
    
		EditTime();
		EditTime( char const * const pnewLabel,
                    TIME &datetime,
                    TimeFormat const format,
                    int const justify=LCD_LEFT_JUSTIFIED,
                    TimeZone const zone=GMT );

        RESULT ChangeChoice( UIEngine &newCallingUI, void (*func)(int Value) );
		
		void SetTime( TIME const &datetime );
                                  
		void	SetLabel( char const * const pnewLabel );                              

		void	SetTimeFormat( TimeFormat const format );
		void	SetTimeZone( TimeZone const zone );
		void	SetJustification( int const justify=LCD_LEFT_JUSTIFIED );

		TIME	GetTime()  
        { 
            VerifyTime();
            return DateTime; 
        }

		char *	GetTimeString( char * const buffer=NULL, int const length=0 );	// returns time buffer and/or  length

        virtual void Redraw();
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived=NULL );

	public: // used by UI Engine
	//Ancestor functionality
        void ResetDimensionDependentData(); //used for updating display parms on the fly
        void Draw( int const PixelOnScreen, FieldLocation const AnchoredFrom, int &TopPixel, 
                   int &HeightinPixels,
                   bool &MoreDataAtTop, bool &MoreDataAtBottom );
    			   //null for toppixel and Height means the caller doesn't care
		void UnDraw();		// process that field is not currently on screen
		bool GiveFocus( int const DirectionMagnitude );
		void TakeFocus();
		int QueryPixelHeight( FieldLocation const StartFrom, int const MaximumRelevant );
        int QueryHeightInArea( FieldLocation const StartFrom );
        int QueryLineHeight( int const MaximumRelevant );
		bool IsFocusVisible(); 

        RESULT HandleInput( MESSAGE const &InputMessage );
    

	protected:   
  
		RESULT HandleCharacter( MESSAGE const& InputMessage);

        //if the parameter is false, the hours will not be converted to 12 hour mode
        //which is confusing when entering digits by hand.
        void DrawTime( bool fFixHoursFor12HourMode = true, bool * pMoreDataAtTop = NULL, bool * pMoreDataAtBottom = NULL );  

		void	ChangeTime( int DirectionMagnitude );
		void	UpdateTime( int newIndex );
		void	GetFieldOffset( int StartAtPixel, int *Offset, int *EndOffset );
		bool	PrevField();		// select previous subfield, if any
		bool	NextField();		// select next subfield, if any

        void    VerifyTime();

        bool FocusFlag;			// true when field has focus
        bool OnLCDFlag;			// true when field is on screen
        bool AMFlag;            // true when the time is AM and 12-HOUR mode is selected

        unsigned char lastGoodMinutes;
        unsigned char lastGoodHours;

        char const * pLabel;
		int WidthofLabel;		// width of label in pixels
   
		typedef enum  
        {
			FIELD_HOUR,
			FIELD_MINUTE,
			FIELD_AMPM
		} TimeField;	
		TimeField Index;		// index into which subfield has current focus
		char TimeBuffer[DATETIME_BUFFER_SIZE];	// current time string

		TIME		DateTime;		// current date/time
		TimeFormat	Format;			// display format: 12 or 24 hour
		int			Justify;
		TimeZone	Zone;		// offset from GMT (+- 12): 1/2 hour incs ??
};

      
#endif	// datetime_h_included
