/****************************************************************************
 Edit.h
 
 Description:
    Edit boxes are fields typically used to input text data.  An edit box 
    has an optional label (as shown below) and is a field that can grow in 
    size.  There is a cursor associated with an edit field and word wrapping 
    is performed automatically.      

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef edit_h_included
#define edit_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
class Edit;

#include "field.h"
#include "buffer.h"
#include "rimctype.h"
#include "utilities.h"



class DLLAccess Edit : public Field
{
	
	public: // used by client

        Edit();
        Edit( char const * const pnewLabel,
             char * const pnewBuffer, 
             int const newLengthofBuffer,
             int const newCharacterOffset =0, 
             int const Justify=LCD_LEFT_JUSTIFIED );
        
        Edit( char const * const pnewLabel,
             int const InitialSize, 
             int const MaximumSize,
             int const Justify=LCD_LEFT_JUSTIFIED );
                                  
        void SetLabel( char const * const pnewLabel );    
        char const * const GetLabel() {return( pLabel );}

		
        void SetBuffer( char * const pnewBuffer, int const newLengthofBuffer );
        void SetBuffer( char const * const pnewBuffer, int const newLengthofBuffer );
        bool SetBuffer( int const InitialSize, int const MaximumSize ); //can fail
		char const * GetBuffer() const;
        int GetBufferLength() const {return( EditBuffer.GetLength() );}
        int GetStringLength() const {return( GetBufferLength()-1 );}

        int GetCursorOffset() const;
        void SetCursorOffset( int const newCharacterOffset = 0 );

        virtual void Redraw();
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );
        virtual bool FocusAtEndOfData();    

        void SetSelection( int Offset, int Length ); //length of 0 clears selection
        void ReplaceSelection( char const * const InsertionString, int const Length=-1 );

        void SetBookmark( int const Offset, 
                          int const BookmarkNumber = Buffer::NUMBER_OF_BOOKMARKS-1 )
            {EditBuffer.SetBookmark( Offset,BookmarkNumber ); return;}
        int GetBookmark(int const BookmarkNumber = Buffer::NUMBER_OF_BOOKMARKS-1 ) const
            { return( EditBuffer.GetBookmark( BookmarkNumber ) ); }

        void SetJustification( int const newJustify=LCD_LEFT_JUSTIFIED );  

        void SetProperties( unsigned long FlagstoSet, unsigned long FlagstoClear=0xFF );//flags from ctype and from below can be bitwise or'd together
        void AddProperties( unsigned long FlagstoSet );
        void ClearProperties( unsigned long FlagstoClear );
        unsigned long GetProperties();

        void Delete( int const Offset, int const Length );
        bool Insert( char const * const pInsertionString, int const Position=-1,
                     int const Length=-1 );
        bool Append( char const * const pInsertionString, int const Length=-1 );

        //Upper 24 bit property definitions //lower 8 bits define character types accepted, see rimctype.h
        enum
        {
            READ_ONLY       = 0x00000100, //setting a field to readonly 
                                            //will make the buffer readonly from that point on, 
                                            //but the reverse is not true
                                            //therefore call SetBuffer again with the new pointer

            CR_TO_ROLL_DOWN             = 0x00000200,
            HANGING_INDENT              = 0x00000400,
            AUTO_SPACE_SUBSTITUTION     = 0x00000800,
            PASSWORD_FIELD              = 0x00001000,    // every character entered will be displayed as an asterisk
            DISABLE_PRESS_HOLD_AUTO_CAP = 0x00002000,
            DISABLE_POSITION_AUTO_CAP   = 0x00004000,
            DISABLE_POSITION_AUTO_PUNC  = 0x00008000,
            DISABLE_AUTO_FORMATTING     = DISABLE_PRESS_HOLD_AUTO_CAP|DISABLE_POSITION_AUTO_CAP|DISABLE_POSITION_AUTO_PUNC,
			SWITCH_ALTED_KEY_AND_KEY    = 0x00010000|DISABLE_AUTO_FORMATTING,
            EMAIL_FIELD                 = AUTO_SPACE_SUBSTITUTION|CTYPE::EMAIL_ADDRESS|DISABLE_AUTO_FORMATTING,
            
            
        };
        
    protected:
        unsigned long Properties;
        int Justify;
        

    

	public: // used by UI Engine
	//Ancestor functionality
        void ResetDimensionDependentData(); //used for updating display parms on the fly    
    
		void Draw( int const PixelOnScreen, FieldLocation const AnchoredFrom, 
                   int &TopPixel, int &HeightinPixels,
                   bool &MoreDataAtTop, bool &MoreDataAtBottom );
        void UnDraw();
        bool GiveFocus( int const DirectionMagnitude );
		void TakeFocus();

        
		int QueryPixelHeight( FieldLocation const StartFrom, int const MaximumRelevant );
        int QueryHeightInArea( FieldLocation const StartFrom );
        int QueryLineHeight( int const MaximumRelevant );
		bool IsFocusVisible();
    
		RESULT HandleInput( MESSAGE const &InputMessage );
    
    public:
        enum
	    {
		    PUNCTUATION_MASK = 0x01,
		    WHITESPACE_MASK = 0x02,
		    CR_MASK	= 0x04,
		    PRONOUN_I_SEPARATOR_MASK = 0x08,
		    PRONOUN_I_MASK = 0x10,
		    ALPHANUM_MASK = 0x20
	    };
	
    protected:

      	protected:
        enum
        {
            HANGING_INDENT_WIDTH = 10
        };
        int HangingIndentWidth;
        enum 
        {
            AP_BOOKMARK_NUMBER,
            AC_BOOKMARK_NUMBER,
            SPACE_SUBSTITUTE_BOOKMARK_NUMBER
        };

        bool APIsUndoneLocation;  //set to true if bookmark is location where AP has been undo
                                  //set to false if bookmark is where AP has been done 
        char ACUndoneCharacter;  //if previous autocap is undone, character is 
                                 //saved here to disable re-expansion 

        void SetInitialSpaceSubstitute(); //sets the space sub bookmark if initial string has @ symbol

        unsigned long BuildPattern();
        void AutoPatternReplace( bool &MustRedraw );  //must redraw is an output to indicate operation needs a redraw 
                                                      //even if the size of current line is OK (may have affected prev line)
        void AutoPatternCheckUndo( bool &MustRedraw, int DeleteStart, int DeleteLength ); //requires DeleteStart and DeleteLength  
                                                                                        //as undo requires recording of information prior
                                                                                        //reversing so action isn't redone
        


        
        void Initialize();
		RESULT HandleScroll( MESSAGE const &InputMessage );
		RESULT HandleCharacter( MESSAGE const &InputMessage );
		RESULT HandleControl( MESSAGE const &InputMessage );
         
		int FindTopFromCursor( int const NumberofLinesAvailable );
		int FindTopFromBottom( int const NumberofLinesAvailable );
		int FindTopFromTop();//essentially "set top"

        void InvertCursor()const;     

        int SetCursorOffsetbyWidth( int const LinetoTarget, int const WidthtoTarget );
		char SwitchKeys( char Key );

		char const * pLabel;
		int WidthofLabel;
    
		
        Buffer EditBuffer;
        
        int LengthofBuffer;        
	    int CurrentOffset;

        int SelectionOffset;
        int SelectionLength;
		
		int MaxLines;
    

    
		int LineWithCursor;
	    int OffsettoLineWithCursor;
		int LengthofLineWithCursor;
		int WidthofLineWithCursor;
        int PositionofLineWithCursor;
        
        void SetCursorParameters();



		int TopLineVisible;
		int OffsettoTopLineVisible;
		int LengthofTopLineVisible;
		int WidthofTopLineVisible;

        int LastOffsetVisible;  //last offset on screen used to determine
                                //if the cursor is visible
        int LastLineVisible;
    
   
		int LineofAnchor;

        bool ValidCursorFlag;
        bool OnLCDFlag;
        bool FocusFlag;

//        bool Scrolling;
        int PixelWidthtoTrackCursor;

    
    
};

class DLLAccess DecimalEdit : public Edit
{
	private:

		char buffer[15];


    public: // used by client
        DecimalEdit();

        DecimalEdit( char * pLabel, int Current, int const Justify=LCD_RIGHT_JUSTIFIED );
        void SetNumber( int Number );
        int GetNumber( void );
		RESULT HandleInput( MESSAGE const &InputMessage );
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived=NULL );

};
      
#endif // edit_h_included