/****************************************************************************
 field.h  -  Base input target that can go on a screen or dialog
 
 Description:
    Fields are screen objects that accept user data. Fields are peers to 
    each other; they are always contained within a screen. Typically 
    rectangular in structure, fields take up the width of the display.

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef field_h_included
#define field_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
class Field;
typedef
enum
{
	TOP_OF_FIELD,       //implies Down
	BOTTOM_OF_FIELD,    //implies Up
	TOP_OF_FOCUS,        //implies Down
	BOTTOM_OF_FOCUS      //implies Up
}
FieldLocation;

typedef
enum
{
    NULL_FIELD,
    EDIT_FIELD = 1,
    DECIMAL_EDIT_FIELD,
    LIST_FIELD,
    STRING_LIST_FIELD,
    SEPARATOR_FIELD,
    CHOICE_FIELD,
    BEEP_CHOICE_FIELD,
    YES_NO_CHOICE_FIELD,
    TOP_BOTTOM_CHOICE_FIELD,
    TABLE_FIELD,
    DATE_FIELD,
    TIME_FIELD
}
FIELDTYPE;

#include "result.h"
#include "manager.h"
#include "pager.h"
#include "helper.h"
#include "linkedlist.h"
class UIEngine;


class DLLAccess Field : public LinkedListEntry
{ 
    public: //called by client 
        virtual void Redraw()= 0;
        virtual bool IsDirty() const;
        virtual void MarkAsClean();
        virtual void MarkAsDirty();
        virtual FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL ) = 0;


	public: //called by field manager
        Field();
        virtual ~Field();
	
		virtual void ResetDimensionDependentData()=0; //used for updatine display parms on the fly
    
        virtual void Draw(int const PixelOnScreen, FieldLocation const AnchoredFrom,
                          int &TopPixel, int &HeightinPixels, 
                          bool &MoreDataAtTop, bool &MoreDataAtBottom ) = 0;
						//null for toppixel and length means the caller doesn't care
        virtual void UnDraw()=0;
 		virtual bool GiveFocus(int const DirectionMagnitude) = 0;
		//positive direction is forward, negative is back
		//0 means leave previous focus
		virtual void TakeFocus()=0;

		virtual int QueryLineHeight(int const MaximumRelevant)=0;
		virtual int QueryPixelHeight(FieldLocation const StartFrom, int const MaximumRelevant)=0;
        virtual int QueryHeightInArea( FieldLocation const StartFrom ) = 0;
		
		virtual bool IsFocusVisible()=0;
		virtual void GetDrawParameters(int &PixelOnScreen, FieldLocation &AnchoredFrom);
    
		virtual void SetFieldManager(FieldManager &newFieldManager);   
        virtual void ClearFieldManager ();

        virtual RESULT ChangeChoice( UIEngine &newCallingUI, void ((*func)(int Value)) = NULL );

		virtual RESULT HandleInput(MESSAGE const &InputMessage)=0;

        void SetTag( int newTag){Tag = newTag; return;}
        int GetTag(){return Tag;}
        bool Scrolling;
        virtual bool FocusAtEndOfData();    
	protected:  
        int Tag;
		FieldManager * pFieldManager;
    
 
		FieldLocation AnchorLocation;
		int PixelinArea;
        bool Dirty;
		bool LineModeChanged;

    
    
};



template< class T >
class SecretField : public T   
{
   private:

      unsigned int password;
      unsigned int input;


   public:

      SecretField( char * secret )
         : input( 0 )
      {
         password  = secret[0] << 24;
         password |= secret[1] << 16;
         password |= secret[2] <<  8;
         password |= secret[3]      ;
         }


		RESULT 
      HandleInput( MESSAGE const & message )
      {
         if(    message.Device == DEVICE_KEYPAD 
             && message.Event  == KEY_DOWN ) {

            if( input == password ) {
               return( T::HandleInput( message ) );
               }

            input <<= 8;
            input |= (byte) message.SubMsg;
            return( CONTINUE );
            }
         
         return( T::HandleInput( message ) );
         }
         
   };

#endif // field_h_included
