/****************************************************************************
 Menu.h   
 
 Description:
    Menus are rectangular containers that are composed of a set of text or 
    menu items. Menus appear on top of screens and cover approximately 
    three-quarters of the screens' width.  Menus should be context sensitive 
    to the data on the associated screen.  There are constant "negative" items 
    (HIDE_MENU) and special registered items (see below).  The application-defined 
    items are also displayed.  A maximum of 32 items can be displyed at once.
     
 Example:
    ...
	char *salutations[6] = 
    {	// array of menu item strings
		"hello", 
		"Hi there",
		"How are you?",
		"Goodbye",
		"Tata",
		"Sayonara"
	};
	...
	Menu MyMenu( salutations, 6 );		    // create menu of salutations
	MyMenu.HideItems( 6 );				    // Hide "Hi There" and "GoodBye" items
	MyUIEngine.ProcessMenu( MyMenu );		// display salutation menu
	while( 1 )							    // process keyboard input until salutation 
                                            //  selected
	{
		RimGetMessage( &msg );
		if( msg.Device == DEVICE_KEYPAD ) 
        {
			result = MyUIEngine.HandleInput( msg );
			if( result == CLICK )			// user has selected item
				break;
		}
	}
	int Item = Menu.GetSelectedItem();		// find item selected
	MyUIEngine.ClearMenu();					// remove menu from screen
	...

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef menu_h_included
#define menu_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_ //UIEngine
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"

class Menu; // class forwarding

#include "pager.h"
#include "result.h"
#include "uiengine.h"

//the following could be static member functions   
void DLLAccess HideTaskNumber( TASK TaskHandleToHide );
void DLLAccess ShowTaskNumber( TASK TaskHandleToShow );
void DLLAccess HideAllTasksFromThread();
void DLLAccess ShowAllTasksFromThread();
int DLLAccess RegisterSystemMenuItem( char const * const pItemString, int TokenToReturn, 
                                      void (*SpecialHandlerFunction)(int Token) );											


class DLLAccess Menu 
{
	public: //used by client

		Menu();					  
		Menu( char const * const * const newMenuStringArray, int const NumberofEntries );
		~Menu();

		// re(create) menu with string array, number of items
		void SetMenuItems( char const * const * const newMenuStringArray, 
                           int const NumberofEntries=-1 );
        
		// specified highlighted item in menu
		void SetSelectedIndex( int const Item );

        // return currently selected menu item
		int GetSelectedIndex() const;

		void SetTopIndex( int const Item );
		int GetTopIndex() const ;

		void SetStartPixelPosition( int StartPixelfromTop );
		int  GetStartPixelPosition() const;
			

		// Make items in list "invisible" to menu
        // not intended to occur while a list is "up" (i.e. do a ProcessMenu)
		void HideItems( int const BitMaptoHide );
		void HideItem( int const Item );
		void ShowItem( int const Item );

    public: //used by UIEngine

        void ResetDimensionDependentData();  
        int GetLCDParametersUsed();
        void SetLCDParametersUsed();
        int GetTaskListVersionUsed();
        void SetTaskListVersionUsed();
    
		void DisplayMenu();
		void ClearMenu();
		virtual RESULT HandleInput( MESSAGE const &InputMessage );

		void SetUIEngine( UIEngine & newCallingUI );
        void ClearUIEngine();

	protected:

		UIEngine * pCallingUIEngine;
	
        int LastLCDParametersUsed;
        int LastTaskListVersionUsed;

        void FindMaximumMenuDimensions();
        void ValidateStartParameters();
		void ClearAndDrawBorder();  
		void DrawStrings();					
        void InvertItem( int const Item);
		void MoveSelector( int const DirectionMagnitude );	
        void MakeSelectorVisible();
		int FindNextTarget( int const Item ) const;
		int FindPrevTarget( int const Item ) const;
		int FindNextVisible( int const Item ) const;
		int FindPrevVisible( int const Item ) const;
		bool IsTarget( int const Item ) const;
		bool IsHidden( int const Item ) const;
		bool IsSeparator( int const Item ) const;
		int	GetItemHeight( int const Item ) const;
		char const * GetItemString(int const Item ) const;
		RESULT HandleClick();
		void MatchCharacter( char CharacterToMatch );

		
		enum  //used for offset
		{
			MINIMUM_VALID_ITEM = -1,
		};

		enum  //used for layout
		{
            SPACE_BEFORE_FIRST_STRING = 1,
			OUTSIDE_WHITESPACE = 1,
			INSIDE_WHITESPACE = 2,
			EXTRA_INSIDE_SPACE_TO_INVERT = 1,
			WIDTH_TO_EDGE_FROM_STRING = 8, //this should be bigger than the width of a scroll character
            MINIMUM_VISIBLE_SCREEN = 10 //must have ten pixels on the screen for reference

		};

	protected: //internal data
		
		char const * const * ItemArray;
		char NumberOfItems; // items in menu
		char TopItemVisible; //top position 
		char BottomItemVisible; //bottom position visiblevisible
		char CurrentItem; // MINIMUM_VALID-1 if no item selected
		
		unsigned long bmHiddenItems; // hidden items: 1 bit/item

		int StartOffset; // start position
		int	HeightUsed; // LCD_HEIGHT - StartHeight is the HeightAvailable, 
						// this value will be less than or equal HeightAvailable
		int WidestString; // calculated to be the widest string in the list
		int	TotalMenuHeight; // total height of the menu

		bool LastAlignedWithTop; //indicator as to whether the strings are flush with the top or bottom


};

#endif // menu_h_included 
