/****************************************************************************
 Radio.h
 
 Description:
    Defines an abstracted interface to the radio modem in the device.

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef RADIO_H
#define RADIO_H

#include <memory.h>

#include "pager.h"

#include "type.h"
#include "utilities.h"
#include "queue.h"

#ifdef DLLAccess
#undef DLLAccess
#endif
#ifdef _TRANSPORT_
#define DLLAccess DLLExport
#else 
#define DLLAccess DLLImport
#endif


#define MAX_PACKET_DATA_LENGTH  MAX_MPAK_DATA_SIZE


typedef int TransmitId;


enum PacketStatus
{
   PACKET_STATUS_PENDING,
   PACKET_STATUS_SENDING,
   };



class Packet;

class DLLAccess RadioNotify : public QueueElement
{
   public:

     /*
      * Transmit
      */
      virtual
      void
      packet_status( Packet & packet,
                     PacketStatus status );


      virtual
      void
      packet_cancelled( Packet & packet );


      virtual
      void
      packet_sent( Packet & packet );


      virtual
      void
      packet_error( Packet & packet,
                    int error );


     /* 
      * Receive
      */
      virtual
      bool
      receive_new_packet( Packet & packet );


      virtual
      void
      receive_returned_packet( Packet & packet,
                               int traffic_state );


     /*
      * Status
      */
      virtual
      void
      radio_status_update( void )
      {
         }


      virtual
      void
      radio_receive_congested( void )
      {
         // free up some resources
         }



     /*
      * Miscellaneous
      */
      RadioNotify *
      get_next_ptr( void )
      {
         return( (RadioNotify *) QueueElement::get_next_ptr() );
         }
   };





class DLLAccess RadioNotifyQueue : public Queue
{
   public:

      RadioNotify *
      get_first_ptr( void )
      {
         return( (RadioNotify *) Queue::get_first_ptr() );
         }

   };






class DLLAccess Packet : public QueueElement
{
   protected:

      RadioNotify * radio_notify_ptr;

      unsigned int remote_address;

      int mpak_type;
      int hpid;

      int   data_length;
      byte  data[ MAX_MPAK_DATA_SIZE ];

		int packet_id;


   public:

      Packet( RadioNotify * init_radio_notify_ptr,
              unsigned int init_remote_address,
              int init_mpak_type,
              int init_hpid = 0,
              int init_data_length = 0,
              byte * data_ptr = NULL );


      Packet( void )
      {
         }


      void
      set_radio_notify( RadioNotify & radio_notify )
      {
         radio_notify_ptr = &radio_notify;
         }


      RadioNotify *
      get_radio_notify( void )
      {
         return( radio_notify_ptr );
         }


      void
      set_info( MPAK_HEADER & header )
      {
         mpak_type = header.MpakType;
         hpid      = header.HPID;
         }


      void
      set_remote_address( unsigned int new_address )
      {
         remote_address = new_address;
         }


      unsigned int
      get_remote_address( void )
      {
         return( remote_address );
         }


      void
      set_mpak_type( int init_mpak_type )
      {
         mpak_type = init_mpak_type;
         }


      int
      get_mpak_type( void )
      {
         return( mpak_type );
         }


      void
      set_hpid( int init_hpid )
      {
         hpid = init_hpid;
         }


      int
      get_hpid( void )
      {
         return( hpid );
         }


      void
      set_data( byte * data_ptr,
                int data_length )
      {
         memcpy( data, data_ptr, data_length );
         }


      byte *
      get_data_ptr( void )
      {
         return( data );
         }


      byte &
      operator [] ( int index )
      {
         return( data[ index ] );
         }


		int
		get_packet_id()
		{
			return packet_id;
			}
		void
		set_packet_id(const int id)
		{
			packet_id = id;
			}


      void
      set_data_length( int init_data_length )
      {
         data_length = init_data_length;
         }


      int
      get_data_length( void )
      {
         return( data_length );
         }


      Packet * 
      get_next_ptr( void )
      {
         return( (Packet *) QueueElement::get_next_ptr() );
         }



     /*
      * radio indications
      */

      void
      packet_cancelled( void )
      {
         radio_notify_ptr->packet_cancelled( *this );
         }


      void
      packet_status( PacketStatus status )
      {
         radio_notify_ptr->packet_status( *this, status );
         }


      void
      packet_sent( void )
      {
         radio_notify_ptr->packet_sent( *this );
         }


      void
      packet_error( int error )
      {
         radio_notify_ptr->packet_error( *this, error );
         }


   };



class DLLAccess PacketQueue : public Queue
{
   public:

      Packet *
      get_first_ptr( void )
      {
         return( (Packet *) Queue::get_first_ptr() );
         }

   };



class DLLAccess Radio
{
   private:

      TASK radio_task_handle;
      unsigned int local_address;

      RadioNotifyQueue radio_notify_queue;
      PacketQueue tx_packet_queue;
      
      TransmitId sending_transmit_id;
      int current_rssi;
      int rssi_threshold; 
      int num_tx_failures;

      bool radio_enabled_flag;
      bool in_tx_coverage_flag;
      bool in_rx_coverage_flag;
      bool tx_sending_flag;
      bool tx_pending_flag;
      bool rx_reception_enabled_flag;
      bool receiving_datagram_flag;

      int last_tx_error;
      int last_tx_result;
      int num_tx_packets;
      int num_tx_attempts;
      int num_tx_sent;
      int num_tx_not_sent;
      int num_tx_errors;
      int num_tx_returned;
      int num_rx_packets;
      int num_rx_congestions;

		// RSSI TX stats
		byte transmit_success[40];				// success at RSSi level -80=>-120 Db
		byte transmit_fail[40];

   public:

      Radio( void );


     /*
      * Client Interface
      */

      void
      reset( void );


      void
      register_radio_notify( RadioNotify * radio_notify_ptr );

      
      void
      send_packet( Packet & packet );


      void
      cancel_packet( Packet & packet );


      bool
      fire_packet( Packet & packet );


      void
      enable_packet_reception( void );


      void
      set_receiving_datagram_flag( bool flag );

      /*
      * Status Interface
      */

      bool
      get_radio_enabled_flag( void )
      {
         return( radio_enabled_flag );
         }

      bool
      get_in_tx_coverage_flag( void )
      {
         return( in_tx_coverage_flag );
         }


      bool
      get_in_rx_coverage_flag( void )
      {
         return( in_rx_coverage_flag );
         }


      bool
      get_tx_sending_flag( void )
      {
         return( tx_sending_flag );
         }


      bool
      get_tx_pending_flag( void )
      {
         return( tx_pending_flag );
         }

      bool
      get_rx_reception_enabled_flag( void )
      {
         return( rx_reception_enabled_flag );
         }

      bool
      get_receiving_datagram_flag( void )
      {
         return receiving_datagram_flag;
         }

      int
      get_current_rssi( void )
      {
         return( current_rssi );
         }



     /*
      * Radio Interface
      */

      void
      process_radio_message( MESSAGE & message );



     /*
      * Timer Interface
      */

      void
      process_radio_timeout( int timer_event=0 );



     /*
      * Misc
      */

      void
      print_debug( void );

      void
      print_rssi( int index );



   private:

      void
      restart( void );


      bool
      start_radio_timer( unsigned int timeout = 0, int timer_event=0);


      int
      transmit_packet( Packet * packet_ptr );


      void
      send_packet( void );


      void
      update_radio_status( void );

		void
		update_rssi_stats( bool packet_sent );

		int
		set_rssi_threshold( void );

      void
      radio_receive_congested( void );

      
      void
      receive_new_packet( Packet & received_packet,
                          int packet_tag );


      void
      receive_returned_packet( Packet & received_packet );


      void
      process_packet_sent_indication( TransmitId id );


      void
      process_packet_not_sent_indication( TransmitId id, int error );


      void
      process_received_packet( int packet_tag );


      void
      check_rssi( int rssi );


      void
      set_radio_enabled_flag( bool flag )
      {
         if( radio_enabled_flag != flag ) {
            radio_enabled_flag = flag;
            update_radio_status();
            }
         }


      void
      set_in_tx_coverage_flag( bool flag )
      {
         if( in_tx_coverage_flag != flag ) {
            in_tx_coverage_flag = flag;
            update_radio_status();
            }
         }

                          
      void
      set_in_rx_coverage_flag( bool flag )
      {
         if( in_rx_coverage_flag != flag ) {
            in_rx_coverage_flag = flag;
            update_radio_status();
            }
         }

                          
      void
      set_tx_sending_flag( bool flag )
      {
         if( tx_sending_flag != flag ) {
            tx_sending_flag = flag;
            update_radio_status();
            }
         }

                         
      void
      set_tx_pending_flag( bool flag )
      {
         if( tx_pending_flag != flag ) {
            tx_pending_flag = flag;
            update_radio_status();
            }
         }


      void
      set_rx_reception_enabled_flag( bool flag )
      {
         if( rx_reception_enabled_flag != flag ) {
            rx_reception_enabled_flag = flag;
            update_radio_status();
            }
         }


   };



/*
 * Radio Functions
 */
extern DLLAccess Radio * get_radio_ptr( void );

extern void put_radio_to_sleep( void );

void log_event( char *letter, int number=0, bool show_zero=false );

#endif