/****************************************************************************
 Status.h
 
 Description:
    Status boxes are rectangular containers that contain application text or 
    text and bitmaps. A status box appears in the middle of the LCD display 
    and is system modal. It does not receive user input yet the foreground 
    application will still receive input while the status box is active. The 
    status box can remain active for a period of time that is definable but 
    since it visually disrupts the screen display, it is typically used only 
    when instant notification is required. 

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef status_h_included
#define status_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
#include "Pager.h"
#include "Area.h"
#include "PredefinedBitmaps.h"

const int   DEVICE_UI_ENGINE         = 10;
const int   EVENT_DISPLAY_STATUS     =  1;
const int   EVENT_KILL_STATUS        =  2;
const int   STATUS_DEFAULT_TIME      = 100;

char const * const UI_ENGINE_PREFIX  = "UI Engine";



class DLLAccess Status
{
	public: //used by client

        //Constructors

        //Default (no string, no bitmap).  SetText() and/or SetBitmap(), and 
        //SetDisplayTime() must be called before displaying the status, or else
        //nothing will be seen on the screen.
		Status();
             
        //These versions of the constructor display the status box immediately.
        //DisplayTime is always in 1/100 second intervals.
        
        //Status box with a string and an information icon
		Status( char const * const pDisplayString, int const DisplayTime = STATUS_DEFAULT_TIME );

        //Status box with a bitmap only (custom or predefined)
		Status( BitMap const * const pBitmap, int const DisplayTime = STATUS_DEFAULT_TIME );
        Status( Bitmaps::PREDEFINED_BITMAP const pdb, int const DisplayTime = STATUS_DEFAULT_TIME );

        //Status box with both a string and a bitmap (custom or predefined).  If you
        //want a string only, you should pass in NULL for pBitmap.
        Status( char const * const pDisplayString, BitMap const * const pBitmap, int const DisplayTime = STATUS_DEFAULT_TIME );
        Status( char const * const pDisplayString, Bitmaps::PREDEFINED_BITMAP const pdb, int const DisplayTime = STATUS_DEFAULT_TIME );

        //Set the text to be displayed in the dialog box.  Word wrapping is done
        //automatically, so there is no need to manually line break the string.
        void SetText( char const * const pDisplayString );

        //Set the bitmap to be displayed in the dialog box.  The first overload
        //allows you to define your own custom bitmap, whereas the second one allows
        //you to use a predefined bitmap.  (See PredefinedBitmaps.h).
        void SetBitmap( BitMap const * const pBitmap );
        void SetBitmap( Bitmaps::PREDEFINED_BITMAP const pdb );

        //Set or get the length of time, in 1/100 second intervals, for which the box 
        //will be displayed.
        void SetDisplayTime( int const DisplayTime );
        int  GetDisplayTime() const;
        
        //Displays the status box system-modally for the length of time specified.
        //After the time has expired, the dialog box will be cleared from the screen.
		void DisplayModalStatus();

	public: //used by UIEngine
        //Draws the status box centred on top of the current screen with the proper 
        //dimensions and with line breaks inserted in the text.  The status box will 
        //NOT be cleared from the screen automatically.
        void DrawStatus();

	protected:
		int             StatusDisplayTime;
        char const *    pStatusString;
        BitMap const *  pStatusBitmap;
        TASK            hUIEngineTask;

};
      
#endif