/****************************************************************************
 Table.h

 Description:
    Tables are fields that present related yet separate items in each cell. 
	The table can be changed programmatically causing changes to smoothly be 
	incorporated into the user's navigation.  When the UI Engine needs to know
    what to display, it will call the pure virtual member function NowDisplaying(),
    passing in the two-byte handle corresponding to the table entry being shown.
    From within this function, the application calls the member function PutCell 
    to actually display the data.

  Example:


 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef table_h_included
#define table_h_included


#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include <limits.h>
#include "field.h"

#define DEF_COLUMN_WIDTH        ((LCD_WIDTH / 4) - 1)


/* General table property flags. */
#define PF_INVERT_ROLL          0x01
#define PF_TOP_BOTTOM_CHECK     0x02
#define PF_LEFT_RIGHT_CHECK     0x04

#define PF_ALIGN_CENTER         0x10
#define PF_ALIGN_RIGHT          0x20

#define PF_MASK                 0x37


/* Cell justification flags. */
#define JF_DEFAULT              0

#define JF_HOR_LEFT             0x1000
#define JF_HOR_CENTERED         0x1100
#define JF_HOR_RIGHT            0x1200
#define JF_HOR_MASK             0x1300

#define JF_VER_TOP              0x2000
#define JF_VER_CENTERED         0x2400
#define JF_VER_BOTTOM           0x2800
#define JF_VER_MASK             0x2C00

#define JF_NONE                 0x3000

#define JF_MASK                 0x3F00


/* Cell border draw flags. */
#define DF_DEFAULT              0

#define DF_NONE                 0x100000

#define DF_LEFT                 0x010000
#define DF_RIGHT                0x020000
#define DF_TOP                  0x040000
#define DF_BOTTOM               0x080000

#define DF_OUTLINE              (DF_LEFT|DF_RIGHT|DF_TOP|DF_BOTTOM)

#define DF_MASK                 0x1F0000


/* Cell highlight flags. */
#define HF_DEFAULT              (HF_HIGHLIGHT_CELL)

#define HF_HIGHLIGHT_NONE       0x01000000
#define HF_HIGHLIGHT_OUTLINE    0x02000000
#define HF_HIGHLIGHT_CELL       0x03000000
#define HF_HIGHLIGHT_CORNERS    0x04000000
#define HF_HIGHLIGHT_ELBOW      0x05000000
#define HF_HIGHLIGHT_CROSS      0x06000000

#define HF_MASK                 0x07000000


class DLLAccess Table : public Field
{
    public: //USED BY A CLIENT
        Table( void );

        Table( int const NumColumns, 
               int const NumRows, 
               int const ColumnWidth  = DEF_COLUMN_WIDTH );

        Table( int const NumColumns, 
               int const NumRows, 
				 bool      AllowBorders,
               int const ColumnWidth  = DEF_COLUMN_WIDTH );

        void SetTableStructure( int const NumColumns, int const NumRows,
                                int const ColumnWidth = DEF_COLUMN_WIDTH );
        void SetTableSize( int const NumColumns, int const NumRows );
        void SetColumnWidth( int const ColumnWidth = DEF_COLUMN_WIDTH );
        void SetRowHeight( int const RowHeight );

        void SetProperties( int const Properties );
        void SetJustification( int const Justify );
        void SetDrawFlags( int const DrawFlags );
        void SetHighlight( int const Highlight );

        void AllowBorders( bool AllowBorders = true );

        void SetTopCell( int const Column, int const Row );
        void SetSelectedCell( int const Column, int const Row );
        void SetSelectedCell( int Offset );

        int  GetNumColumns( void ) const;
        int  GetNumRows( void ) const;
        void GetNumCells( int *oNumColumns, int *oNumRows );

        int  GetTopColumn( void ) const;
        int  GetTopRow( void ) const;
        void GetTopCell( int *Column, int *Row );

        int  GetSelectedColumn( void ) const;
        int  GetSelectedRow( void ) const;
        void GetSelectedCell( int *Column, int *Row );

        void InsertColumn( int const Column, bool Redraw = true );
        void InsertRow( int const Row, bool Redraw = true );
        void DeleteColumn( int const Column, bool Redraw = true );
        void DeleteRow( int const Row, bool Redraw = true );

        void MoveHighlight( const int nNewColOnScreen, 
                            const int nNewRowOnScreen, 
                            const int nOldColOnScreen = -1, 
                            const int nOldRowOnScreen = -1 );

        virtual void Redraw( void );
        virtual void Redraw( int const Column, int const Row );

        virtual void NowDisplaying( int const Column, int const Row ) = 0;
        virtual bool IsCellSelectable( int const Column, int const Row );

        void PutCell( char const * const Str, 
                      int const          ColumnWidth  = LCD_WIDTH,
                      int const          JustifyFlags = JF_DEFAULT, 
                      int const          StrLength    = INT_MAX,
                      int const          BorderFlags  = DF_DEFAULT );

        void PutCell( BitMap const * const Ptr, 
                      int const            Xpos = 0, 
                      int const            Ypos = 0,
                      int const            BorderFlags  = DF_DEFAULT );
        
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );


    public: //USED BY A FIELD MANAGER
        //ancestor functions

        virtual void Draw( int const DrawAtPixel, FieldLocation const AnchoredFrom,
                           int &TopPixel, int &HeightInPixels, bool &MoreDataAtTop, 
                           bool &MoreDataAtBottom );

        void ResetDimensionDependentData(); //used for updating display parms on the fly    

        void UnDraw( void );
        bool GiveFocus( int const Offset );
        void TakeFocus( void );
        int  QueryLineHeight( int const MaxRelevant );
        int  QueryHeightInArea( FieldLocation const StartFrom );
        int  QueryPixelHeight( FieldLocation const StartFrom,
                              int const MaxRelevant );
        bool   IsFocusVisible( void );
        RESULT HandleInput( MESSAGE const &Msg );


    protected:

        RESULT HandleScroll( MESSAGE const &Msg );

        int     NumCols;
        int     NumRows;
        int     NumCells;

        int     NumColsOnScreen;
        int     NumRowsOnScreen;

        int     ColWithSelection;
        int     RowWithSelection;

        int     StartingColToDraw;
        int     StartingRowToDraw;

        XYPoint PointForCell;
        XYPoint PointForString;
        
        bool    OnLCDFlag;
        bool    FocusFlag;
        bool    AllowBordersFlag;
        
        int     ColumnWidth;
        int     RowHeight;
        int     OffsetFromLeftOfScreen;
        
        int     Properties;
        int     CurrentCellBorders;
};


#endif  /* table_h_included */
