/****************************************************************************
 Transport.h

 Description:
    This file describes the interface to the transport layer.

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef TRANSPORT_H
#define TRANSPORT_H

#include "DataBuffer.h"
#include "type.h"
#include "radio.h"

#ifdef DLLAccess
#undef DLLAccess
#endif
#ifdef _TRANSPORT_
#define DLLAccess DLLExport
#else 
#define DLLAccess DLLImport
#endif

enum DatagramStatus {
   DATAGRAM_STATUS_PENDING,      // these should be the same as PacketStatus
   DATAGRAM_STATUS_SENDING,
   };


typedef int TransportId;



class Datagram;


class DLLAccess TransportNotify : public QueueElement
{
   public:

     /*
      * Transmit indications
      */

      virtual
      void
      datagram_status( Datagram & datagram,
                       DatagramStatus status );


      virtual
      void
      datagram_cancelled( Datagram & datagram );


      virtual
      void
      datagram_sent( Datagram & datagram );


      virtual
      void
      datagram_error( Datagram & datagram,
                      int radio_error );                 


      virtual
      void
      datagram_returned( TransportId transport_id, 
                         int traffic_state,             
                         Datagram * datagram_ptr );     // may be NULL


     /*
      * Receive Indications
      */

      virtual
      bool
      receive_datagram( Datagram & datagram ); 

             

     /*
      * misc
      */

      TransportNotify *
      get_next_ptr( void )
      {
         return( (TransportNotify *) QueueElement::get_next_ptr() );
         }

   };




class DLLAccess TransportNotifyQueue : public Queue
{
   public:

      TransportNotify *
      get_first_ptr( void )
      {
         return( (TransportNotify *) Queue::get_first_ptr() );
         }

   };



                               


class DLLAccess Datagram : public DataBuffer
{
   private:
      
      TransportNotify * transport_notify_ptr;

      TransportId transport_id;
      unsigned int remote_address;

      int local_port;
      int remote_port;

      int compression_id;
      int encryption_id;

      int datagram_length;
      int cursor_position;

   
   public:

      Datagram( TransportNotify * init_transport_notify_ptr,
                int init_ram_max_data_length = 500,
                int init_record_length = 500 );         

                          
      Datagram( TransportNotify * init_transport_notify_ptr,
                HandleType db_handle,
                int datagram_length,
                int init_cursor_position = 0 );
                            

      Datagram( TransportNotify * init_transport_notify_ptr,
                byte * datagram_ptr,
                int datagram_length,
                int init_cursor_position = 0 );

      
      void
      reset( void );


      int
      get_datagram_length( void )
      {
         return( get_data_length() );
         }


      TransportId
      get_transport_id( void )
      {
         return( transport_id );
         }


      void
      set_transport_id( TransportId new_transport_id )
      {
         transport_id = new_transport_id;
         }


      unsigned int
      get_remote_address( void )
      {
         return( remote_address );
         }


      void
      set_remote_address( unsigned int address )
      {
         remote_address = address;
         }


      int
      get_local_port( void )
      {
         return( local_port );
         }


      void
      set_local_port( int new_local_port )
      {
         local_port = new_local_port;
         }


      int
      get_remote_port( void )
      {
         return( remote_port );
         }


      void
      set_remote_port( int new_remote_port )
      {
         remote_port = new_remote_port;
         }


      int
      get_compression_id( void )
      {
         return( compression_id );
         }


      void
      set_compression_id( int new_compression_id )
      {
         compression_id = new_compression_id;
         }


      int
      get_encryption_id( void )
      {
         return( encryption_id );
         }


      void
      set_encryption_id( int new_encryption_id )
      {
         encryption_id = new_encryption_id;
         }



     /*
      * transport indications
      */

      virtual
      void
      datagram_status( DatagramStatus status )
      {
         transport_notify_ptr->datagram_status( *this, status );
         }


      virtual
      void
      datagram_cancelled( void )
      {
         transport_notify_ptr->datagram_cancelled( *this );
         }


      virtual
      void
      datagram_sent( void )
      {
         transport_notify_ptr->datagram_sent( *this );
         }


      virtual
      void
      datagram_error( int radio_error )                      // radio error
      {
         transport_notify_ptr->datagram_error( *this, radio_error );
         }


     /*
      * misc
      */
      Datagram *
      get_next_ptr( void )
      {
         return( (Datagram *) QueueElement::get_next_ptr() );
         }
   };



   
class DLLAccess DatagramQueue : public Queue
{
   public:

      Datagram *
      get_first_ptr( void )
      {
         return( (Datagram *) Queue::get_first_ptr() );
         }

   };




class DLLAccess Transport : public RadioNotify
{
   protected:

      DatagramQueue tx_datagram_queue;
      TransportNotifyQueue transport_notify_queue;

      int receive_enabled_lock;

      unsigned int default_gateway_address;

      bool receiving_datagram_flag;


   public:

      Transport( void )
         : receive_enabled_lock( 0 )
         , default_gateway_address( 0 )
         , receiving_datagram_flag( true )
      {
         }


     /*
      * Client Interface
      */

      virtual
      void
      reset( void );


      void
      register_transport_notify( TransportNotify * transport_notify_ptr )
      {
         transport_notify_queue.insert( transport_notify_ptr );
         }


      void
      enable_receive( void ) 
      {
         if( --receive_enabled_lock < 0 ) {
            receive_enabled_lock = 0;   // oops
            }
         }


      void
      disable_receive( void ) 
      {
         receive_enabled_lock++;
         }


      virtual
      void
      set_default_gateway_address( unsigned int gateway_address )
      {
         default_gateway_address = gateway_address;
         }


      virtual
      TransportId
      send_datagram( Datagram & datagram ) = 0;


      virtual
      void
      cancel_datagram( Datagram & datagram )
      {
         }


      virtual
      void
      cancel_datagram( TransportId transport_id )
      {
         }


      

     /*
      * private
      */

      virtual
      void
      initialize( void )
      {
         }


      virtual
      void
      power_up( void )
      {
         }


      virtual
      void
      Transport::
      process_rtc( void );


      virtual
      void
      print_debug( void );



   protected:

      Datagram *
      find_datagram_ptr( TransportId transport_id );

      
      void
      datagram_returned( TransportId transport_id, 
                         int traffic_state,            
                         Datagram * datagram_ptr );   

      
      bool
      receive_datagram( Datagram & datagram );


      void
      set_receiving_datagram_flag( bool flag=true );

   };


extern DLLAccess Transport * get_transport_ptr( void );


#endif